@@ -6,18 +6,17 @@
  */
 
 
-#include<types.h>
-#include<lib.h>
-#include<lock.h>
-#include<device.h>
-#include<wait.h>
-#include<interrupt.h>
-#include<signal.h>
-#include<keymap.h>
-#include<term.h>
-#include<errno.h>
-#include<mm.h>
-#include<console.h>
+#include"types.h"
+#include"lib.h"
+#include"lock.h"
+#include"device.h"
+#include"wait.h"
+#include"interrupt.h"
+#include"signal.h"
+#include"keymap.h"
+#include"term.h"
+#include"errno.h"
+#include"console.h"
 
 
 /***************************************************************************************
@@ -36,12 +35,12 @@ TERM_CTL termCtl;
  * ҥץƱGIDΥץ˥ʥ롣
  * parameters : begin process,groupe ID,signal
  */
-static void sendSignalToGrp(PROC *parent,pid_t pgid,int signal)
+static void sendSignalToGrp(PROC *parent,gid_t gid,int signal)
 {
 	PROC *p;
 
 
-	for(p=parent;(p=getNextProc(p,pgid))!=NULL;)
+	for(p=parent;(p=getNextProc(p,gid))!=NULL;)
 		sendSignal(p,signal);
 }
 
@@ -65,15 +64,15 @@ static struct termios termio={
 static inline void initTerm()
 {
 	/* termiosꡣ */
-	termio.c_cc[VINTR]=	CINTR;
-	termio.c_cc[VQUIT]=	CQUIT;
-	termio.c_cc[VSTART]=CSTART;
-	termio.c_cc[VSTOP]=	CSTOP;
-	termio.c_cc[VSUSP]=	CDSUSP;
-	termio.c_cc[VEOF]=	CEOF;
-	termio.c_cc[VEOL]=	CEOL;
-	termio.c_cc[VERASE]=CERASE;
-	termio.c_cc[VKILL]=	CKILL;
+	termio.c_cc[VINTR]=	'c'&0x1f;
+	termio.c_cc[VQUIT]=	'/'&0x1f;
+	termio.c_cc[VSTART]='q'&0x1f;
+	termio.c_cc[VSTOP]=	's'&0x1f;
+	termio.c_cc[VSUSP]=	'z'&0x1f;
+	termio.c_cc[VEOF]=	'd'&0x1f;
+	termio.c_cc[VEOL]=	0;
+	termio.c_cc[VERASE]='h'&0x1f;
+	termio.c_cc[VKILL]=	'u'&0x1f;
 }
 
 
@@ -83,7 +82,7 @@ static inline void initTerm()
  */
 static inline void isForeground()
 {
-	if(get_current_task()->pgid!=termCtl.foregrnd)
+	if(get_current_task()->gid!=termCtl.foregrnd)
 		del_from_schedule(TASK_SIGNAL_WAIT);
 }
 
@@ -112,7 +111,7 @@ void releaseTerm(PROC *proc)
 
 enum{
 	/* VGA Video ram. */
-	VRAM_BASE_ADDR=0xb8000,	/* Start address of video memory */
+	VRAM_BASE_ADDR=0xb8000,		/* Start address of video memory */
 	VRAM_SIZE=0x8000,		/* Size of video memory */
 
 	/* CRT IO registers. */
@@ -127,20 +126,19 @@ enum{
 	SCR_COLUMNS=80,			/* Number of columns. */
 	SCR_LAST=				/* Video ram last address. */
 		VRAM_SIZE/2-SCR_LINES*SCR_COLUMNS-SCR_COLUMNS*3,
-
-	TAB=8,					/* Tab characters. */
 };
 
 
 typedef struct{
 	int viewTop;	/* ӥǥ꡼β̤γϰ֡ */
 	int start;		/* ϰ֡ */
+	int cursor;		/* ֡ */
 }SCREEN;
 
 
 /* PRIVATE */
 static char *vram=(char*)VRAM_BASE_ADDR;
-static SCREEN screen;
+static SCREEN screen={0,0,0};
 
 
 /*
@@ -150,9 +148,9 @@ static SCREEN screen;
 static inline void updateCursor()
 {
 	outb(CTL_ADDR,CURSOR_ADDR_HIGH);
-	outb(CTL_DATA,screen.start>>8);
+	outb(CTL_DATA,screen.cursor>>8);
 	outb(CTL_ADDR,CURSOR_ADDR_LOW);
-	outb(CTL_DATA,screen.start);
+	outb(CTL_DATA,screen.cursor);
 }
 
 
@@ -183,19 +181,19 @@ static void scrollup()
 	int i,last;
 
 
-	screen.viewTop=ROUNDUP(screen.start-SCR_COLUMNS*SCR_LINES+1,SCR_COLUMNS);
+	screen.viewTop=ROUNDUP(screen.cursor-SCR_COLUMNS*SCR_LINES+1,SCR_COLUMNS);
 
 	if(screen.viewTop>=SCR_LAST)
 	{
 		/* Video ramƬ˰ư */
 		p=&vram[screen.viewTop*2];
-		last=(screen.start-screen.viewTop)*2;
+		last=(screen.cursor-screen.viewTop)*2;
 		for(i=0;i<last;i+=2)vram[i]=p[i];
 
 		/* Video ram0ꥢ */
 		for(;i<(SCR_LAST+SCR_COLUMNS*SCR_LINES)*2;i+=2)vram[i]=0;
 
-		screen.start-=screen.viewTop;
+		screen.start=screen.cursor-=screen.viewTop;
 		screen.viewTop=0;
 	}
 
@@ -240,15 +238,15 @@ static inline void putNumIntoScreen(char *buf,int size)
 /* Left cursor. */
 static inline void leftCursor(int n)
 {
-	screen.start-=n;
+	screen.start=screen.cursor-=n;
 	updateCursor();
 }
 
 /* Right cursor. */
 static inline void rightCursor(int n)
 {
-	screen.start+=n;
-	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
+	screen.start=screen.cursor+=n;
+	if(screen.cursor-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
 		scrollup();
 	updateCursor();
 }
@@ -256,24 +254,15 @@ static inline void rightCursor(int n)
 /* Carriage retun. */
 static inline void carriageRetun()
 {
-	screen.start=ROUNDDOWN(screen.start,SCR_COLUMNS);
+	screen.start=screen.cursor=ROUNDDOWN(screen.cursor,SCR_COLUMNS);
 	updateCursor();
 }
 
 /* New line. */
 static inline void newLine()
 {
-	screen.start=ROUNDUP(screen.start+1,SCR_COLUMNS);
-	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
-		scrollup();
-	updateCursor();
-}
-
-/* Tab. */
-static inline void tab()
-{
-	screen.start=ROUNDUP(screen.start+1,TAB);
-	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
+	screen.start=screen.cursor=ROUNDUP(screen.cursor+1,SCR_COLUMNS);
+	if(screen.cursor-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
 		scrollup();
 	updateCursor();
 }
@@ -305,9 +294,8 @@ static void initScreen()
  *
  ***************************************************************************************/
 
-enum{
+ enum{
 	CONSOLE_BUF_SIZE=90,	/* Line buffer size. */
-	CONSOLE_EOF=0xff,		/* EOF character. */
 };
 
 
@@ -428,13 +416,8 @@ static inline void newLineBuf()
  */
 static int Eof()
 {
-	if(lbuf.lineTop==lbuf.last)	/* ʸʤ */
-	{
-		lbuf.buf[lbuf.last++]=CONSOLE_EOF;
-		lbuf.lineTop=lbuf.crt=lbuf.last;
-		return 1;
-	}
-
+	if(lbuf.lineTop==lbuf.last)return 1;	/* ʸʤ */
+	lbuf.lineTop=lbuf.crt=lbuf.last;
 	return 0;
 }
 
@@ -443,7 +426,7 @@ static int Eof()
  * PUBLIC
  * Check console buffer.
  * parameters : user buffer,max copy size
- * return : ϥХȿ or EOF=0 or Ϥʤ=-1
+ * return : ϥХȿ
  */
 static int readConsoleBuf(uchar *user_buf,size_t size)
 {
@@ -452,20 +435,9 @@ static int readConsoleBuf(uchar *user_buf,size_t size)
 
 	if(termio.c_lflag&ICANON)	/* Canonical mode. */
 	{
-		if(lbuf.last==0)return -1;		/* Ϥʤ */
-
-		if((uchar)*lbuf.buf==CONSOLE_EOF)
-		{
-			*lbuf.buf='\0';
-			lbuf.lineTop=lbuf.crt=lbuf.last=0;
-
-			return 0;
-		}
-
 		i=0;
-		while (lbuf.buf[i++]!='\n')
-			if (i >= lbuf.last)
-				return -1;
+		while(i<lbuf.last)
+			if(lbuf.buf[i++]=='\n')break;
 		size=(size>i)?i:size;
 		memcpy(user_buf,lbuf.buf,size);
 		memcpy(lbuf.buf,lbuf.buf+i,lbuf.last-i);
@@ -482,24 +454,6 @@ static int readConsoleBuf(uchar *user_buf,size_t size)
 }
 
 
-/*
- * PUBLIC
- * 󥽡Хåեɤ߹߲ǽ
- * return : no=0 or yes=1
- */
-static int checkConsoleBuf()
-{
-	int i;
-
-
-	if((uchar)*lbuf.buf==CONSOLE_EOF)return 1;
-	for(i=0;i<lbuf.last;++i)
-		if(lbuf.buf[i]=='\n')return 1;
-
-	return 0;
-}
-
-
 /***************************************************************************************
  *
  * ܡ
@@ -511,9 +465,8 @@ enum{
 };
 
 
-/* PUBLIC */
-static PROC *waitProc;	/* Ԥץ */
-static int waitLock;	/* ѥå */
+/* GLOBAL */
+static PROC_LINK waitLink={&waitLink,&waitLink};	/* Ԥץ󥯡 */
 
 
 /*
@@ -728,24 +681,12 @@ static int keyboardHnadler(uchar key_coad)
  */
 static int keyboardIntrHandler()
 {
-	int rest=0;
-
-
 	if(keyboardHnadler(inb(KEYBOARD_OUTPUT_REG)))
 	{
-		enter_spinlock(&waitLock);
-		{
-			if(waitProc!=NULL)
-			{
-				forceSendSignal(waitProc,SIGCONT);
-				waitProc=NULL;
-				rest=1;
-			}
-		}
-		exit_spinlock(&waitLock);
+		wakeProc(&waitLink);
+		return 1;
 	}
-
-	return rest;
+	return 0;
 }
 
 
@@ -764,21 +705,31 @@ static int keyboardIntrHandler()
 int writeConsole(void *buf,size_t size,size_t start)
 {
 	char *str=(char*)buf;
+	int cursol;
 	int i;
 
 
-	for(i=0;i<size;++i)
+	cursol=0;
+	i=0;
+	for(;i<size;++i)
 	{
 		if(str[i]>=' ')
+		{
 			putIntoScreen(str[i]);
+			++cursol;
+		}
     	else if(str[i]=='\n')
+    	{
     		newLine();
+    		cursol=0;
+    	}
     	else if(str[i]=='\r')
+    	{
     		carriageRetun();
-    	else if(str[i]=='\t')
-    		tab();
+    		cursol=0;
+    	}
     }
-    rightCursor(0);
+    rightCursor(cursol);
 
 	return i;
 }
@@ -790,7 +741,6 @@ int writeConsole(void *buf,size_t size,size_t start)
  *
  ***************************************************************************************/
 
-
 static int open()
 {
 	return 0;
@@ -809,17 +759,13 @@ static int read(void *buf,size_t size,size_t start)
 		wait_task();
 	}
 
-	/* Ϥ뤫󥽡Хåեǧ */
-	enter_spinlock(&waitLock);
-	if((len=readConsoleBuf(buf,size))==-1)
-	{
-		waitProc=get_current_task();			/* Ԥ */
-		exit_spinlock(&waitLock);
-		sigWait();								/* ȡ */
 
+	/* Ϥ뤫󥽡Хåեǧ */
+	if((len=readConsoleBuf(buf,size))==0)
+	{
+		waitProc(&waitLink);			/* Ԥ */
 		len=readConsoleBuf(buf,size);
 	}
-	else exit_spinlock(&waitLock);
 
 	return len;
 }
@@ -838,12 +784,11 @@ static int write(void *buf,size_t count,size_t x)
 }
 
 
-int ioctl(int cmd,...)
+int ioctl(int prm_num,int cmd,...)
 {
 	uint *arg=(uint*)&cmd+1;
 	PROC *proc,*grp_proc;
-	pid_t pgid;
-	struct winsize *win;
+	gid_t grp;
 
 
 	switch(cmd)
@@ -852,62 +797,33 @@ int ioctl(int cmd,...)
 			if(termCtl.ctlProc==NULL)
 			{
 				termCtl.ctlProc=get_current_task();
-				termCtl.foregrnd=termCtl.ctlProc->pgid;
+				termCtl.foregrnd=termCtl.ctlProc->gid;
 				termCtl.ctlProc->ctlterm=&termCtl;
 			}
 			else return -EPERM;
 			break;
 		case TIOSPGRP:
-			pgid=(gid_t)*arg;
+			grp=(gid_t)*arg;
 			proc=get_current_task();
 			if(proc!=termCtl.ctlProc)return -ENOTTY;
-			if(proc->pgid!=pgid)
+			if(proc->gid!=grp)
 			{
 				/* Ϳ줿롼פƱåΤΤ */
-				if((grp_proc=getNextProc(proc,pgid))==NULL)return -EINVAL;
+				if((grp_proc=getNextProc(proc,grp))==NULL)return -EINVAL;
 				if((grp_proc->ctlterm==NULL)||(((TERM_CTL*)grp_proc->ctlterm)->ctlProc!=proc))return -EPERM;
 			}
-			termCtl.foregrnd=pgid;
+			termCtl.foregrnd=grp;
 
 			break;
 		case TIOGPGRP:
 			if(get_current_task()->ctlterm!=&termCtl)return -ENOTTY;
 			return termCtl.foregrnd;
-		case TIOCGWINSZ:
-			win=(struct winsize*)*arg;
-			if(checkMem(win)==-1)return -EFAULT;
-			win->ws_row=SCR_LINES;
-			win->ws_col=SCR_COLUMNS;
-			break;
 	}
 
 	return 0;
 }
 
 
-/* return : 0 or 1=Ѳ */
-static int select(int flag,int polling)
-{
-	int rest;
-
-
-	if(flag==FD_WRITE)return 1;
-
-	/* polling mode. */
-	if(polling)
-		return checkConsoleBuf();
-
-	enter_spinlock(&waitLock);
-	{
-		if((rest=checkConsoleBuf())==0)
-			waitProc=get_current_task();	/* Ԥ */
-	}
-	exit_spinlock(&waitLock);
-
-	return rest;
-}
-
-
 /***************************************************************************************
  *
  * 
@@ -916,7 +832,7 @@ static int select(int flag,int polling)
 
 /* PRIVATE,Device interface */
 static DEV_INFO devInfo={
-	"console",0,0,0,open,read,write,ioctl,select
+	"console",0,0,0,open,read,write,ioctl,
 };
 
 
