@@ -10,8 +10,7 @@
 #define console_h
 
 
-#include<types.h>
-#include<proc.h>
+#include"types.h"
 
 
 #ifndef ASM_FILE
