@@ -203,15 +203,13 @@ static DEV_INODE *getNextInode(DEV_INODE *dind)
  * parameters : device inode,stat store buffer
  * return : 0 or error=-1
  */
-int get_devstat(int din,DEV_STAT *buf)
+int get_dev_stat(int din,DEV_STAT *buf)
 {
 	if(dinode[din].ref_count==0)return -1;
 
 	buf->prt_type=dinode[din].pt_type;
-	buf->prt_num=dinode[din].pt_num;
 	buf->sect_size=dinode[din].dev->sector_size;
 	buf->all_sect=dinode[din].blocks;
-	buf->name=dinode[din].dev->name;
 
 	return 0;
 }
@@ -330,13 +328,13 @@ typedef struct CACHE_TABLE{
 	struct CACHE_TABLE *prev;		/* Cache chain previous */
 	struct CACHE_TABLE *write_next;	/* ǥХؽ񤭹߹ꥹ */
 	struct CACHE_TABLE *write_prev; /* ǥХؽ񤭹߹ꥹ */
-	void               *addr;		/* Cache address */
-	uint                block;		/* Device block index */
-	uchar               din;		/* Device inode number */
-	uchar               bitmap;		/* ǡ¸ߥӥåȥޥå */
-	uchar               dirty;		/* ߤե饰 */
-	uchar volatile      writing;	/* ǥХؽ񤭹߹ե饰 */
-	int                 gate;		/* ԥå */
+	void *addr;						/* Cache address */
+	uint block;						/* Device block index */
+	uchar din;						/* Device inode number */
+	uchar bitmap;					/* ǡ¸ߥӥåȥޥå */
+	uchar dirty;					/* ߤե饰 */
+	uchar volatile writing;			/* ǥХؽ񤭹߹ե饰 */
+	int gate;						/* ԥå */
 }CACHE_TABLE;
 
 typedef struct{
@@ -352,9 +350,6 @@ static PROC *write_task=NULL;						/* ٱ񤭹ѥ */
 static CACHE_TABLE delay_list=						/* ٱ񤭹ߥꥹȤƬ */
 	{NULL,NULL,NULL,NULL,&delay_list,&delay_list,NULL,0,0,0,0,0,0};
 static int cacheLock=0;								/* å󥯹ѥåȡ */
-static uchar bitSects[]={							/* PRIVATE ȤΥӥåȥޥå͡ */
-	0,0x1,0x3,0x7,0xf,0x1f,0x3f,0x7f,0xff
-};
 
 
 /*
@@ -363,9 +358,8 @@ static uchar bitSects[]={							/* PRIVATE ȤΥӥåȥޥå͡ */
  * parameters : CACHE_TABLE *p
  * return : 0 or error number
  */
-static int readDevice(CACHE_TABLE *p,uchar reqmap)
+static int readDevice(CACHE_TABLE *p)
 {
-	uchar map;
 	int i,k;
 
 
@@ -373,24 +367,23 @@ static int readDevice(CACHE_TABLE *p,uchar reqmap)
 	{
 		if(dinode[p->din].dev->read(p->addr,CACHE_BLOCK_SIZE/dinode[p->din].dev->sector_size,p->block+dinode[p->din].begin_blk)<0)
 			return  -EIO;
-		p->bitmap=0xff;
 	}
 	else
 	{
 		/* ɤ߹ޤƤʤս򥭥åɤ߹ࡣ */
-		map=~p->bitmap&reqmap;
 		for(i=0;i<CACHE_BLOCK_SIZE/CACHE_UNIT_SIZE;++i)
-			if((map&(1<<i))!=0)
+			if((p->bitmap&(1<<i))==0)
 			{
 				for(k=i+1;k<CACHE_BLOCK_SIZE/CACHE_UNIT_SIZE;++k)
-					if((map&(1<<k))==0)break;
+					if((p->bitmap&(1<<k))!=0)break;
 				if(dinode[p->din].dev->read((char*)p->addr+i*CACHE_UNIT_SIZE,k-i,p->block+i+dinode[p->din].begin_blk)<0)
 					return -EIO;
 				i=k;
 			}
-		p->bitmap|=reqmap;
 	}
 
+	p->bitmap=0xff;
+
 	return 0;
 }
 
@@ -547,19 +540,16 @@ static CACHE_TABLE *search_cache(int din,size_t begin)
 			if(p==cache_old)
 			{
 				cache_old=p->prev;
-				add_delay_list(cache_old);
+				if(cache_old->dirty)add_delay_list(cache_old);
 			}
-			else
-			{
-				/* 󥯤ֿ֤ˤ롣 */
-				p->next->prev=p->prev;
-				p->prev->next=p->next;
-				p->next=cache_old->next;
-				p->prev=cache_old;
-				cache_old->next=p;
-				p->next->prev=p;
-			}
-			add_delay_list(cache_old->prev);
+
+			/* 󥯤ֿ֤ˤ롣 */
+			p->next->prev=p->prev;
+			p->prev->next=p->next;
+			p->next=cache_old->next;
+			p->prev=cache_old;
+			cache_old->next=p;
+			p->next->prev=p;
 
 			return p;
 		}
@@ -586,7 +576,7 @@ static CACHE_TABLE *getEmptyCache(int din,size_t begin)
 	cache_old=p->prev;
 
 	/* μθŤå᤹ */
-	add_delay_list(cache_old->prev);
+	if(cache_old->prev->dirty)add_delay_list(cache_old->prev);
 
 	/* ϥå󥯤 */
 	p->hprev->hnext=p->hnext;
@@ -654,53 +644,42 @@ static void writeBackAllCache()
  */
 int read_cache(int din,void *buf,size_t sector_num,size_t begin)
 {
-	int sector_size = dinode[din].dev->sector_size;	/* ǥХ */
-	int sectors = CACHE_BLOCK_SIZE / sector_size;	/* å֥åˤϤ륻 */
-	int cache_mask = sectors - 1;					/* åƬ򻻽Ф뤿Υӥåȥޥ */
-	int rest_num = sector_num;						/* Ĥž */
+	int sector_size=dinode[din].dev->sector_size;	/* ǥХ */
+	int sectors=CACHE_BLOCK_SIZE/sector_size;		/* å֥åˤϤ륻 */
+	int cache_mask=sectors-1;						/* åƬ򻻽Ф뤿Υӥåȥޥ */
+	int rest_num=sector_num;						/* Ĥž */
 	int start,sect_num;
 	CACHE_TABLE *p;
-	union{
-		void *pv;
-		uint ui;
-	}value;
 
 
 	/* ǥ¤Υå */
-	if (begin > dinode[din].blocks)
-		return -EIO;
-	if (begin + sectors > dinode[din].blocks)
-		return -EIO;
+	if(begin>dinode[din].blocks)return -EIO;
+	if(begin+sectors>dinode[din].blocks)return -EIO;
 
-	start = begin & cache_mask;
-	begin &= ~cache_mask;
-	sect_num = (rest_num < sectors - start)? rest_num : sectors - start;
+	start=begin&cache_mask;
+	begin&=~cache_mask;
+	sect_num=(rest_num<sectors-start)?rest_num:sectors-start;
 	for(;;)
 	{
 		/* å򸡺 */
 		enter_spinlock(&cacheLock);
 		{
-			if ((p = search_cache(din,begin)) == NULL)
-				p = getEmptyCache(din,begin);
+			if((p=search_cache(din,begin))==NULL)p=getEmptyCache(din,begin);
 		}
 		exit_spinlock(&cacheLock);
 
 		/* å׵᥻ʤХǥХɤ߹ࡣ */
-		if (((p->bitmap >> start) & bitSects[sect_num]) != bitSects[sect_num])
-			if (readDevice(p,bitSects[sect_num] << start) < 0)
-				return -EIO;
+		if((p->bitmap|(((1<<sect_num)-1)<<start))!=p->bitmap)
+			if(readDevice(p)<0)return -EIO;
 
 		/* Хåե˥ԡ롣 */
 		memcpy(buf,(char*)p->addr+start*sector_size,sect_num*sector_size);
 
-		if ((rest_num -= sect_num) == 0)
-			break;
-		value.pv = buf;
-		value.ui += sect_num * sector_size;
-		buf = value.pv;
-		begin += sectors;
-		sect_num = (rest_num < sectors)? rest_num : sectors;
-		start = 0;
+		if((rest_num-=sect_num)==0)break;
+		(char*)buf+=sect_num*sector_size;
+		begin+=sectors;
+		sect_num=(rest_num<sectors)?rest_num:sectors;
+		start=0;
 	}
 
 	return sector_num;
@@ -715,51 +694,41 @@ int read_cache(int din,void *buf,size_t sector_num,size_t begin)
  */
 int write_cache(int din,void *buf,size_t sector_num,size_t begin)
 {
-	int sector_size = dinode[din].dev->sector_size;	/* ǥХ */
-	int sectors = CACHE_BLOCK_SIZE / sector_size;	/* å֥åˤϤ륻 */
-	int cache_mask = sectors - 1;					/* åƬ򻻽Ф뤿Υӥåȥޥ */
-	int rest_num = sector_num;						/* Ĥž */
+	int sector_size=dinode[din].dev->sector_size;	/* ǥХ */
+	int sectors=CACHE_BLOCK_SIZE/sector_size;		/* å֥åˤϤ륻 */
+	int cache_mask=sectors-1;						/* åƬ򻻽Ф뤿Υӥåȥޥ */
+	int rest_num=sector_num;						/* Ĥž */
 	int start,sect_num;
 	CACHE_TABLE *p;
-	union{
-		void *pv;
-		uint ui;
-	}value;
 
 
 	/* ǥ¤Υå */
-	if (begin > dinode[din].blocks)
-		return -EIO;
-	if (begin + sectors > dinode[din].blocks)
-		return -EIO;
+	if(begin>dinode[din].blocks)return -EIO;
+	if(begin+sectors>dinode[din].blocks)return -EIO;
 
-	start = begin & cache_mask;
-	begin &= ~cache_mask;
-	sect_num = (rest_num < sectors - start)? rest_num : sectors-start;
+	start=begin&cache_mask;
+	begin&=~cache_mask;
+	sect_num=(rest_num<sectors-start)?rest_num:sectors-start;
 	for(;;)
 	{
 		/* å򸡺 */
 		enter_spinlock(&cacheLock);
 		{
-			if ((p = search_cache(din,begin)) == NULL)
-				p = getEmptyCache(din,begin);
+			if((p=search_cache(din,begin))==NULL)p=getEmptyCache(din,begin);
 		}
 		exit_spinlock(&cacheLock);
 
 		/* å˥ԡ롣 */
-		memcpy((char*)p->addr + start * sector_size,buf,sect_num * sector_size);
+		memcpy((char*)p->addr+start*sector_size,buf,sect_num*sector_size);
 
-		p->bitmap |= bitSects[sect_num] << start;
-		p->dirty = 1;
+		p->bitmap|=((1<<sect_num)-1)<<start;
+		p->dirty=1;
 
-		if ((rest_num -= sect_num) == 0)
-			break;
-		value.pv = buf;
-		value.ui += sect_num * sector_size;
-		buf = value.pv;
-		begin += sectors;
-		sect_num = (rest_num < sectors)? rest_num : sectors;
-		start = 0;
+		if((rest_num-=sect_num)==0)break;
+		(char*)buf+=sect_num*sector_size;
+		begin+=sectors;
+		sect_num=(rest_num<sectors)?rest_num:sectors;
+		start=0;
 	}
 
 	return sector_num;
@@ -876,6 +845,7 @@ static int open(const char *path,int dev,uint dir,int flag,uint *inode)
 	int i;
 
 
+
 	/* ץγǧ */
 	if(isMaxDevOpen()==0)return -EMDEV;
 
@@ -936,24 +906,22 @@ static int open(const char *path,int dev,uint dir,int flag,uint *inode)
 
 		/* ĥѡƥ󥯤򤿤ɤ롣 */
 		beg_blk=next_blk=pt[i].lba_begin;
-		for(i = PT_NUM;;)
+		for(i=PT_NUM;;)
 		{
-			if ((rest = dinf->read(buf,1,next_blk)) < 0)
-				goto ERR;
-			if (++i == pt_num)
-				break;
-			if (pt[1].type != PT_TYPE_EXT)
+			if((rest=dinf->read(buf,1,next_blk))<0)goto ERR;
+			if(++i==pt_num)break;
+			if(pt[1].type!=PT_TYPE_EXT)
 			{
-				rest = -ENODEV;
+				rest=-ENODEV;
 				goto ERR;
 			}
-			next_blk = beg_blk+pt[1].lba_begin;
+			next_blk=beg_blk+pt[1].lba_begin;
 		}
-		dind = makeInode(name,pt[0].lba_begin+next_blk,pt[0].size,pt[0].type,pt_num,dinf);
+		dind=makeInode(name,pt[0].lba_begin+next_blk,pt[0].size,pt[0].type,pt_num,dinf);
 	}
 	kfree(buf);
-	dind->ref_count = 1;
-	*inode = getInodeNum(dind);
+	dind->ref_count=1;
+	*inode=getInodeNum(dind);
 
 	return DEVICE_FILE;
 ERR:
@@ -965,7 +933,7 @@ ERR:
 /*
  * parameters : device inode,inode block
  */
-static int close(int din,uint inode,int type)
+static int close(int din,uint inode)
 {
 	if(--dinode[inode].ref_count<0)dinode[inode].ref_count=0;
 
@@ -982,10 +950,6 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	int rest=size;		/* ĤžХȿ */
 	int beg_sect;		/* žϤΥֹ */
 	int s,t;
-	union{
-		void *pv;
-		uint ui;
-	}value;
 
 
 	if((sect_siz=dinode[inode].dev->sector_size)==0)
@@ -1000,7 +964,7 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	if((s=begin%sect_siz)!=0)
 	{
 		if((tmp=(char*)kmalloc(sect_siz))==NULL)return -1;
-		if(read_cache(inode,tmp,1,beg_sect)!=1)return 0;
+		if(read_cache(din,tmp,1,beg_sect)!=1)return 0;
 		t=sect_siz-s;
 		if(t>size)
 		{
@@ -1009,15 +973,13 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 			return size;
 		}
 		memcpy(buf,tmp+s,t);
-		value.pv = buf;
-		value.ui += t;
-		buf = value.pv;
+		(char*)buf+=t;
 		beg_sect+=1;
 		rest-=t;
 	}
 
 	s=rest/sect_siz;
-	if((t=read_cache(inode,buf,s,beg_sect))<s)
+	if((t=read_cache(din,buf,s,beg_sect))<s)
 	{
 		kfree(tmp);
 		return size-rest+t*sect_siz;
@@ -1028,7 +990,7 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	{
 		t=s*sect_siz;
 		if((tmp==NULL)&&((tmp=(char*)kmalloc(sect_siz))==NULL))return -1;
-		if(read_cache(inode,tmp,1,beg_sect+s)!=1)
+		if(read_cache(din,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
@@ -1051,11 +1013,6 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	int rest=size;		/* ĤžХȿ */
 	int beg_sect;		/* žϤΥֹ */
 	int s,t;
-	union{
-		void *pv;
-		uint ui;
-	}value;
-
 
 
 	if((sect_siz=dinode[inode].dev->sector_size)==0)
@@ -1070,25 +1027,23 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	if((s=begin%sect_siz)!=0)
 	{
 		if((tmp=(char*)kmalloc(sect_siz))==NULL)return -1;
-		if(read_cache(inode,tmp,1,beg_sect)!=1)return 0;
+		if(read_cache(din,tmp,1,beg_sect)!=1)return 0;
 		if((t=sect_siz-s)>size)
 		{
 			memcpy(tmp+s,buf,size);
-			if(write_cache(inode,tmp,1,beg_sect)!=1)return 0;
+			if(write_cache(din,tmp,1,beg_sect)!=1)return 0;
 			kfree(tmp);
 			return size;
 		}
 		memcpy(tmp+s,buf,t);
-		if(write_cache(inode,tmp,1,beg_sect)!=1)return 0;
-		value.pv = buf;
-		value.ui += t;
-		buf = value.pv;
+		if(write_cache(din,tmp,1,beg_sect)!=1)return 0;
+		(char*)buf+=t;
 		beg_sect+=1;
 		rest-=t;
 	}
 
 	s=rest/sect_siz;
-	if((t=write_cache(inode,buf,s,beg_sect))<s)
+	if((t=write_cache(din,buf,s,beg_sect))<s)
 	{
 		kfree(tmp);
 		return size-rest+t*sect_siz;
@@ -1099,13 +1054,13 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	{
 		t=s*sect_siz;
 		if((tmp==NULL)&&((tmp=(char*)kmalloc(sect_siz))==NULL))return -1;
-		if(read_cache(inode,tmp,1,beg_sect+s)!=1)
+		if(read_cache(din,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
 		}
 		memcpy(tmp,(char*)buf+t,rest-t);
-		if(write_cache(inode,tmp,1,beg_sect+s)!=1)
+		if(write_cache(din,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
@@ -1170,21 +1125,19 @@ static int readdir(int din,uint inode,uint *block,uint *index,char *name)
 }
 
 
-static int stat(int din,uint inode,int type,FSTAT *fstat)
+static int stat(int din,uint inode,FSTAT *fstat)
 {
-	fstat->st_dev=    inode;
-	fstat->st_ino=    inode;	/* ȤꤢinodeƤ */
-	fstat->st_mode=   0;
-	fstat->st_nlink=  0;
-	fstat->st_uid=    0;
-	fstat->st_gid=    0;
-	fstat->st_size=   dinode[inode].blocks*dinode[inode].dev->sector_size;
-	fstat->st_atime=  0;
-	fstat->st_mtime=  0;
-	fstat->st_ctime=  0;
+	fstat->st_dev=inode;
+	fstat->st_ino=0;
+	fstat->st_mode=0;
+	fstat->st_nlink=0;
+	fstat->st_uid=0;
+	fstat->st_gid=0;
+	fstat->st_size=dinode[inode].blocks*dinode[inode].dev->sector_size;
+	fstat->st_atime=0;
+	fstat->st_mtime=0;
+	fstat->st_ctime=0;
 	fstat->st_blksize=dinode[inode].dev->sector_size;
-	fstat->st_blocks= 0;
-	fstat->st_rdev=   0;
 
 	return 0;
 }
@@ -1201,11 +1154,11 @@ static int ioctl(int din,uint inode,int prm_num,int cmd,...)
 	switch(prm_num)
 	{
 		case 0:
-			return dinode[inode].dev->ioctl(cmd);
+			return dinode[inode].dev->ioctl(0,cmd);
 		case 1:
-			return dinode[inode].dev->ioctl(cmd,*arg);
+			return dinode[inode].dev->ioctl(1,cmd,*arg);
 		case 2:
-			return dinode[inode].dev->ioctl(cmd,*arg,*(arg+1));
+			return dinode[inode].dev->ioctl(2,cmd,*arg,*(arg+1));
 		default:
 			return 0;
 	}
@@ -1230,7 +1183,7 @@ static int rename(int din,uint old_dinode,const char *old_path,uint new_dinode,c
 }
 
 
-static int creat(const char *path,int din,uint dir_inode,int mode,uint *inode)
+static int creat(const char *path,int din,uint dir_inode,int mode)
 {
 	return -EDNOFUNC;
 }
@@ -1247,7 +1200,7 @@ static int delete(const char *path,int din,uint dir_inode,int type)
 	return -EDNOFUNC;
 }
 
-static int chattr(int din,uint inode,int type,uint mode,uint uid,uint gid,uint atime,uint mtime)
+static int chattr(int din,uint inode,uint mode,uint uid,uint gid,uint atime,uint mtime)
 {
 	return -EDNOFUNC;
 }
@@ -1258,22 +1211,9 @@ static int mklink(int din,uint oldinode,uint newind,const char *newpath)
 }
 
 
-static int select(uint inode,int flag,int polling)
-{
-	return dinode[inode].dev->select(flag,polling);
-}
-
-
-static int fsstat(int din,struct statfs *buf)
-{
-	return -EDNOFUNC;
-}
-
-
 static FS dev_fs={
 	"dev_fs",mount,umount,open,close,read,write,ioctl,rename,creat,opendir,mkdir,stat,delete,readdir,
-	chattr,mklink,select,fsstat,
-};
+	chattr,mklink};
 
 
 /*
@@ -1299,8 +1239,8 @@ int sys_ttyname(int fdnum,char *buf)
 	if((fd=getFd(fdnum))==NULL)return -EBADF;
 
 	/* DEV_INFOΥ饹ȥ֥åʥС0Υ饯ǥХǧ롣 */
-	dev=dinode[fd->vent->inode].dev;
-	if(dev->sector_size!=0)return -ENOTTY;
+	dev=dinode[fd->vent->din].dev;
+	if(dev->last_blk!=0)return -ENOTTY;
 
 	memcpy(buf,dev->name,strlen(dev->name)+1);
 
