@@ -15,15 +15,13 @@
 #include<mm.h>
 #include<proc.h>
 #include<lock.h>
+#include<fs.h>
 #include<test.h>
 #include<errno.h>
 #include<interrupt.h>
 #include<time.h>
-#include<signal.h>
-#include<device.h>
 #include<net/net.h>
 #include<share/utime.h>
-#include<fs.h>
 
 
 /***************************************************************************************
@@ -111,8 +109,8 @@ static const char *searchEntry(const char *path,VENTRY *entry,VENTRY **buf)
 
 	if(rest==0)
 	{
-		for(path+=entry->len;*path=='/';++path);
-		return path;
+		if(path[entry->len]=='/')return &path[entry->len+1];
+		return &path[entry->len];
 	}
 
 	return NULL;
@@ -277,16 +275,13 @@ static inline void linkVentry(VENTRY *vent)
  */
 static inline int unlinkVentry(VENTRY *vent)
 {
-	int cnt;
-
-
-	if((cnt=--vent->refCount)<=0)
+	if(--vent->refCount<=0)
 	{
 		delUpEntry(vent);
 		return 0;
 	}
 
-	return cnt;
+	return vent->refCount;
 }
 
 
@@ -333,8 +328,7 @@ static int mountEntry(VENTRY *entry,FS *fs,int din,uint block)
 	/* ޥȥȥ꡼κ */
 	if((mount_ent=(VENTRY*)kmalloc(sizeof(VENTRY)+entry->len))==NULL)return -ENOMEM;
 	memset(mount_ent,0,sizeof(VENTRY));
-	mount_ent->next=entry->next;
-	mount_ent->prev=entry->prev;
+	mount_ent->next=mount_ent->prev=mount_ent;
 	mount_ent->inode=block;
 	mount_ent->parent=entry->parent;
 	mount_ent->fs=fs;
@@ -424,9 +418,9 @@ static int getVentObj(SEARCH_PARAM *start,SEARCH_PARAM *end)
 
 
 	enter_spinlock(&spinLock);
-	{
+	{	
 		if((rest=searchPath(start,end))==0)
-			end->vent->refCount+=1;
+			end->vent->refCount+=1;			
 	}
 	exit_spinlock(&spinLock);
 
@@ -462,98 +456,6 @@ static void releaseVentObj(VENTRY *vent)
 }
 
 
-/*
- * PUBLIC
- * struct statfs˳Ǽ롣
- * return : 0 or error number
- */
-static int getFsstat(VENTRY *vent,struct statfs *buf)
-{
-	int rest;
-	DEV_STAT devstat;
-
-
-	if((rest=vent->fs->fsstat(vent->din,buf))<0)
-		return rest;
-	get_devstat(vent->din,&devstat);
-
-	if(devstat.prt_num>0)
-		sprintk(buf->f_mntfromname,"/dev/%s%d",devstat.name,devstat.prt_num);
-	else
-		sprintk(buf->f_mntfromname,"/dev/%s",devstat.name,devstat);
-	copyCurrentPath(buf->f_mntonname,MNAMELEN,vent);
-
-	return 0;
-}
-
-
-/*
- * PRIVATE
- * ǥ쥯ȥޥȥȥ꡼θ
- * parameters : VENTRY,struct statfs,struct statfsΰ,struct statfs̵ե饰
- */
-static int searchMountDir(VENTRY *vent,struct statfs *buf,int buf_cnt,int flag)
-{
-	int rest,count;
-	VENTRY *vent_begin;
-
-
-	if(buf_cnt<=0)return 0;
-	vent_begin=vent;
-	count=buf_cnt;
-
-	do
-	{
-		/* ޥȤꡣ */
-		if(vent->mount!=NULL)
-		{
-			while(vent->mount!=NULL)vent=vent->mount;
-			if(flag)
-				if(getFsstat(vent,buf)==0)
-				{
-					buf+=1;
-					count-=1;
-				}	
-		}
-
-		if(vent->child!=NULL)
-		{
-			rest=searchMountDir(vent->child,buf,count,flag);
-			buf+=rest;
-			count-=rest;
-		}
-	}
-	while((vent=vent->next)!=vent_begin);
-
-	return buf_cnt-count;
-}
-
-
-/*
- * PUBLIC
- * ޥȥȥ꡼򸡺롣
- * parameters : struct statfs,buf count
- * return : struct statfsسǼ or ޥȿstruct statfs=NULL
- */
-static int searchMount(struct statfs *buf,int buf_cnt)
-{
-	int rest,flag;
-
-
-	if(buf_cnt<=0)return 0;
-
-	flag=(buf==NULL)?0:1;
-	
-	/* rootե륷ƥࡣ */
-	if(flag==1)
-		getFsstat(rootEnt,buf);
-
-	rest=searchMountDir(rootEnt->child,buf+1,buf_cnt-1,flag);
-
-	return rest+1;
-}
-
-
 /************************************************************************************
  *
  * < ե륷ƥ饹 >
@@ -821,29 +723,27 @@ static int unlinkFd(FILE_STRUCT *fstruct,int num)
 	FS *fs;
 
 
-	fd = fstruct->fd[num];
-	fd->refCount -= 1;
-	if (fd->form == FD_FORM_NETSOKET)
+	fd=fstruct->fd[num];
+	fd->refCount-=1;
+	if(fd->form==FD_FORM_NETSOKET)
 	{
-		if (fd->refCount <= 0)
+		if(fd->refCount<=0)
 			closeSocket((SOCKET*)fd);
 	}
 	else
 	{
-		vent = fd->vent;
-		if (fd->refCount <= 0)
+		vent=fd->vent;
+		if(fd->refCount<=0)
 		{
 			/* õե饰ǧ */
-			if(fd->accessMode & O_DEL)delFromFd(fd);
+			if(fd->accessMode&O_DEL)delFromFd(fd);
 
 			kfree(fd);
 		}
 		fs=vent->fs;
 		din=vent->din;
 		block=vent->inode;
-
-		if (unlinkVentry(vent) == 0)
-			rest = fs->close(din,block,vent->type);
+		if(unlinkVentry(vent)==0)rest=fs->close(din,block);
 	}
 
 	deleteFd(fstruct,num);
@@ -864,17 +764,15 @@ FILE_STRUCT *cpy_file_struct(FILE_STRUCT *fstruct)
 	FILE_STRUCT *cp_struct;
 
 
-	cp_struct = kmalloc(sizeof(FILE_STRUCT));
-	if (cp_struct == NULL)
-		return NULL;
+	if((cp_struct=kmalloc(sizeof(FILE_STRUCT)))==NULL)return NULL;
 	memcpy(cp_struct,fstruct,sizeof(FILE_STRUCT));
 
 	/* եǥץλȥ󥿤1䤹 */
-	for (cnt = fstruct->fd_count,i = 0; cnt > 0; ++i)
-		if (cp_struct->fd[i] != NULL)
+	for(cnt=fstruct->fd_count,i=0;cnt>0;++i)
+		if(cp_struct->fd[i]!=NULL)
 		{
 			++cp_struct->fd[i]->refCount;
-			if (cp_struct->fd[i]->form == FD_FORM_LOCALFILE)
+			if(cp_struct->fd[i]->form==FD_FORM_LOCALFILE)
 				linkVentry(cp_struct->fd[i]->vent);
 			--cnt;
 		}
@@ -895,13 +793,12 @@ void releaseFileStruct(FILE_STRUCT *fstruct)
 
 
 	/* Release all of file descriptors. */
-	for (i = 0;i < MAX_FILE_OPEN; ++i)
+	for(i=0;i<MAX_FILE_OPEN;++i)
 	{
-		if (fstruct->fd[i] != NULL)
+		if(fstruct->fd[i]!=NULL)
 		{
 			unlinkFd(fstruct,i);
-			if (fstruct->fd_count <= 0)
-				break;
+			if(fstruct->fd_count<=0)break;
 		}
 	}
 
@@ -1045,20 +942,6 @@ enum{
 	FS_FORM_DIR= 0x4000,	/* directory */
 	FS_FORM_CHR= 0x2000,	/* character device */
 	FS_FORM_FIFO=0x1000,	/* fifo */
-	FS_FORM_MASK=0xF000,	/* format mask */
-
-	/* ե饰 */
-	FS_IRUSR=0x0100,		/* Read permission, owner. */
-	FS_IWUSR=0x0080,		/* Write permission, owner. */
-	FS_IXUSR=0x0040,		/* Execute/search permission, owner. */
-	FS_IRWXG=0x0038,		/* Read, write, execute/search by group. */
-	FS_IRGRP=0x0020,		/* Read permission, group. */
-	FS_IWGRP=0x0010,		/* Write permission, group. */
-	FS_IXGRP=0x0008,		/* Execute/search permission, group. */
-	FS_IRWXO=0x0007,		/* Read, write, execute/search by others. */
-	FS_IROTH=0x0004,		/* Read permission, others. */
-	FS_IWOTH=0x0002,		/* Write permission, others. */
-	FS_IXOTH=0x0001,		/* Execute/search permission, others. */
 };
 
 
@@ -1072,7 +955,7 @@ static inline VENTRY *getTopVdir(const char **path)
 {
 	if(**path=='/')
 	{
-		while(*(++*path)=='/');
+		*path+=1;
 		return rootEnt;
 	}
 	else
@@ -1125,13 +1008,13 @@ int exec_open(const char *path,OPEN_F *open_f)
 			rest=mkEntry(end.path,end.vent);
 		}
 		exit_spinlock(&spinLock);
-
+*/
 		if(rest<0)
 		{
-			start.vent->fs->close(start.vent->din,start.vent->inode,NORMAL_FILE);
+			start.vent->fs->close(start.vent->din,start.vent->inode);
 			goto END;
 		}
-*/
+
 		open_f->din=start.vent->din;
 		open_f->inode=inode;
 		open_f->fs=start.vent->fs;
@@ -1266,12 +1149,11 @@ int sys_lseek(int fd_num,int offset,int flag)
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
 	vent=fd->vent;
-	vent->fs->stat(vent->din,vent->inode,vent->type,&fstat);
+	vent->fs->stat(vent->din,vent->inode,&fstat);
 	switch(flag)
 	{
 		case SEEK_CUR:
 			rest=fd->offset;
-			break;
 		case SEEK_SET:
 			if((rest+=offset)>fstat.st_size)return -EOVERFLOW;
 			break;
@@ -1290,7 +1172,7 @@ int sys_lseek(int fd_num,int offset,int flag)
 int sys_creat(const char *path,int flag,int mode)
 {
 	int rest;
-	uint inode;
+	int block;
 	SEARCH_PARAM start,end;
 	VENTRY *vent;
 	F_DSC *fd;
@@ -1312,10 +1194,10 @@ int sys_creat(const char *path,int flag,int mode)
 		goto END;
 	}
 
-	if (flag & O_FIFO)
-		mode |= FS_FORM_FIFO;
-	else mode=(mode & ~FS_FORM_MASK) | FS_FORM_REG | FS_IRUSR | FS_IWUSR;
-	if((rest=start.vent->fs->creat(start.path,start.vent->din,start.vent->inode,mode,&inode))<0)goto END;
+	if(mode&O_FIFO)mode|=FS_FORM_FIFO;
+	else mode|=FS_FORM_REG;
+	if((rest=start.vent->fs->creat(start.path,start.vent->din,start.vent->inode,mode))<0)goto END;
+	block=rest;
 
 	enter_spinlock(&spinLock);
 	{
@@ -1324,13 +1206,13 @@ int sys_creat(const char *path,int flag,int mode)
 	exit_spinlock(&spinLock);
 	if(rest<0)
 	{
-		start.vent->fs->close(start.vent->din,start.vent->inode,NORMAL_FILE);
+		start.vent->fs->close(start.vent->din,start.vent->inode);
 		goto END;
 	}
 
 	vent=(VENTRY*)rest;
-	vent->inode=inode;
-	if (flag & O_FIFO)
+	vent->inode=block;
+	if(mode&O_FIFO)
 	{
 		if((vent->wait=kmalloc(sizeof(WAIT_QUEUE)))==NULL)goto END;
 		memset(vent->wait,0,sizeof(WAIT_QUEUE));
@@ -1358,8 +1240,8 @@ END:
  */
 int sys_open(const char *path,int flag,int mode)
 {
-	uint inode,dummy;
-	int rest,type;
+	uint inode;
+	int rest;
 	SEARCH_PARAM start,end;
 	VENTRY *vent;
 	F_DSC *fd;
@@ -1377,33 +1259,26 @@ int sys_open(const char *path,int flag,int mode)
 
 	if(*end.path=='\0')
 	{
-		if(end.vent->type==DIRECTORY)
+		if(end.vent->type!=DIRECTORY)
 		{
-			if(flag!=O_RDONLY)
-			{
-				rest=-EISDIR;
-				goto END;
-			}
-			if(end.vent->refCount==1)
-			{
-				if((rest=start.vent->fs->opendir(start.path,start.vent->din,start.vent->inode,&inode,&dummy))<0)
-					goto END;
-				end.vent->inode=inode;
-			}
+			linkVentry(end.vent);
+			vent=end.vent;
+		}
+		else
+		{
+			rest=-EISDIR;
+			goto END;
 		}
-		linkVentry(end.vent);
-		vent=end.vent;
 	}
 	else
 	{
 		rest=start.vent->fs->open(start.path,start.vent->din,start.vent->inode,flag&O_TRUNC,&inode);
-		if ((rest == -ENOENT) && (flag & O_CREAT))
+		if((rest==-ENOENT)&&(flag&O_CREAT))
 		{
-			mode=(mode & ~FS_FORM_MASK) | FS_FORM_REG | FS_IRUSR | FS_IWUSR;
-			rest=start.vent->fs->creat(start.path,start.vent->din,start.vent->inode,mode,&inode);
+			mode|=FS_FORM_REG;
+			rest=start.vent->fs->creat(start.path,start.vent->din,start.vent->inode,mode);
 		}
-		if (rest < 0)goto END;
-		type=rest;
+		if(rest<0)goto END;
 
 		enter_spinlock(&spinLock);
 		{
@@ -1412,19 +1287,19 @@ int sys_open(const char *path,int flag,int mode)
 		exit_spinlock(&spinLock);
 		if(rest<0)
 		{
-			start.vent->fs->close(start.vent->din,start.vent->inode,NORMAL_FILE);
+			start.vent->fs->close(start.vent->din,start.vent->inode);
 			goto END;
 		}
 
 		vent=(VENTRY*)rest;
 		vent->inode=inode;
-		if (flag & O_FIFO)
+		if(mode&O_FIFO)
 		{
 			if((vent->wait=kmalloc(sizeof(WAIT_QUEUE)))==NULL)goto END;
 			memset(vent->wait,0,sizeof(WAIT_QUEUE));
 			vent->type=FIFO_FILE;
 		}
-		else vent->type=type;
+		else vent->type=NORMAL_FILE;
 	}
 
 	/* եǥץϿ롣 */
@@ -1453,6 +1328,7 @@ int sys_close(int fd_num)
 
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
+
 	fstruct=(FILE_STRUCT*)get_current_task()->file_struct;
 	return unlinkFd(fstruct,fd_num);
 }
@@ -1515,8 +1391,8 @@ int sys_write(int fd_num,void *buf,size_t size)
 	else
 	{
 		rest=vent->fs->write(vent->din,vent->inode,buf,size,fd->offset);
-		if(rest > 0)
-			fd->offset+=rest;
+		if(rest==-1)return -1;
+		fd->offset+=rest;
 	}
 
 	return rest;
@@ -1534,6 +1410,7 @@ int sys_mkdir(const char *path,int mode)
 	start.vent=getTopVdir(&path);
 	start.path=path;
 	if((rest=getVentObj(&start,&end))<0)return rest;
+
 	if(*end.path=='\0')
 	{
 		rest=-EEXIST;
@@ -1618,7 +1495,7 @@ int sys_opendir(const char *path)
 			linkVentry(end.vent);
 			vent=end.vent;
 			if((rest=start.vent->fs->opendir(end.path,vent->din,vent->inode,&inode,&block))<0)
-			goto MKFD;
+			goto END;
 		}
 		else
 		{
@@ -1640,7 +1517,7 @@ int sys_opendir(const char *path)
 				exit_spinlock(&spinLock);
 				if(rest<=0)
 				{
-					start.vent->fs->close(start.vent->din,start.vent->inode,DIRECTORY);
+					start.vent->fs->close(start.vent->din,start.vent->inode);
 					goto END;
 				}
 				vent=(VENTRY*)rest;
@@ -1656,7 +1533,6 @@ int sys_opendir(const char *path)
 		}
 	}
 
-MKFD:
 	/* ǥ쥯ȥꥹȥ꡼κ */
 	if((dir=kmalloc(sizeof(DIR_STRAEM)))!=NULL)
 	{
@@ -1665,6 +1541,7 @@ MKFD:
 		dir->index=0;
 	}
 	else rest=-ENOMEM;
+
 END:
 	releaseVentObj(end.vent);
 
@@ -1682,7 +1559,6 @@ int sys_readdir(int dir_num,char *name)
 	if(dir->fd.form!=FD_FORM_DIRSTREAM)return -EBADF;
 
 	vent=dir->fd.vent;
-
 	return (vent->fs)->readdir(vent->din,dir->block,&dir->block,&dir->index,name);
 }
 
@@ -1720,7 +1596,7 @@ int sys_rename(const char *old_path,const char *new_path)
 	/* եõ */
 	new_start.vent=getTopVdir(&new_path);
 	new_start.path=new_path;
-	if((rest=getVentObj(&new_start,&new_end))<0)goto END;
+	if((rest=getVentObj(&old_start,&old_end))<0)goto END;
 
 	if(old_start.vent->din==new_start.vent->din)
 	{
@@ -1746,7 +1622,7 @@ int sys_chdir(const char *path)
 	SEARCH_PARAM start,end;
 	VENTRY *vent;
 
-	
+
 	if(*path=='\0')return -ENOENT;
 
 	/* ǥ쥯ȥõ */
@@ -1758,12 +1634,6 @@ int sys_chdir(const char *path)
 	{
 		if(*end.path=='\0')
 		{
-			if(end.vent->refCount==1)
-			{
-				if((rest=start.vent->fs->opendir(start.path,start.vent->din,start.vent->inode,&inode,&dummy))<0)
-					goto ERR;
-				end.vent->inode=inode;
-			}
 			linkVentry(end.vent);
 			vent=end.vent;
 		}
@@ -1779,7 +1649,7 @@ int sys_chdir(const char *path)
 
 				if(rest<=0)
 				{
-					start.vent->fs->close(start.vent->din,start.vent->inode,DIRECTORY);
+					start.vent->fs->close(start.vent->din,start.vent->inode);
 					goto ERR;
 				}
 				vent=(VENTRY*)rest;
@@ -1816,8 +1686,6 @@ int sys_fchdir(int fd_num)
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
 	if(fd->vent->type!=DIRECTORY)return -ENOTDIR;
 
-	linkVentry(fd->vent);
-	
 	/* ե빽¤Τ˿ȥǥ쥯ȥϿ */
 	setCurrentDir(fd->vent);
 
@@ -1851,9 +1719,9 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 {
 	uint inode;
 	int rest;
-	int type;
 	SEARCH_PARAM start,end;
 	F_DSC *fd;
+	uint dummy;
 
 
 	if(checkMem(buf)==-1)return -EFAULT;
@@ -1862,12 +1730,10 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 	{
 		if((fd=getFd(fd_num))==NULL)return -EBADF;
 		buf->st_crtpos=fd->offset;
-		return fd->vent->fs->stat(fd->vent->din,fd->vent->inode,fd->vent->type,buf);
+		return fd->vent->fs->stat(fd->vent->din,fd->vent->inode,buf);
 	}
 	else
 	{
-		if(checkMem((char*)path)==-1)return -EFAULT;
-
 		buf->st_crtpos=0;
 
 		start.vent=getTopVdir(&path);
@@ -1875,23 +1741,20 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 		if((rest=getVentObj(&start,&end))<0)return rest;
 
 		if((*end.path=='\0')&&(end.vent->inode!=0))
-			rest=end.vent->fs->stat(end.vent->din,end.vent->inode,end.vent->type,buf);
+			rest=end.vent->fs->stat(end.vent->din,end.vent->inode,buf);
 		else
 		{
-			if((rest=start.vent->fs->open(start.path,start.vent->din,start.vent->inode,0,&inode))<0)
-				goto END;
-			type=rest;	
-			rest=start.vent->fs->stat(start.vent->din,inode,type,buf);
-			start.vent->fs->close(start.vent->din,inode,type);
+			if((rest=start.vent->fs->open(start.path,start.vent->din,start.vent->inode,0,&inode))==-EISDIR)
+				rest=start.vent->fs->opendir(start.path,start.vent->din,start.vent->inode,&inode,&dummy);
+			if(rest<0)goto END;
+
+			rest=start.vent->fs->stat(start.vent->din,inode,buf);
+			start.vent->fs->close(start.vent->din,inode);
 		}
 	}
 END:
 	releaseVentObj(end.vent);
 
-/************** ̤ **************************/
-	buf->st_flags=0;
-/*****************************************************/
-
 	return rest;
 }
 
@@ -1923,7 +1786,6 @@ static void umountAllFs(VENTRY *ventry)
 int sys_umount_root()
 {
 	umountAllFs(rootEnt);
-	rootEnt->fs->umount(rootEnt->din);
 
 	return 0;
 }
@@ -1960,7 +1822,7 @@ int sys_fcntl(int fd_num,int cmd,uint arg)
 }
 
 
-static int chattr(FS *fs,int din,uint inode,int type,int cmd,uint attr)
+static int chattr(FS *fs,int din,uint inode,int cmd,uint attr)
 {
 	int rest;
 	PROC *proc;
@@ -1969,22 +1831,22 @@ static int chattr(FS *fs,int din,uint inode,int type,int cmd,uint attr)
 
 	/* Change attribute. */
 	proc=get_current_task();
-	if((rest=fs->stat(din,inode,type,&stat))<0)return rest;
+	if((rest=fs->stat(din,inode,&stat))<0)return rest;
 	switch(cmd)
 	{
 		case CHATTR_MODE:
 			if((proc->uid==0)||(proc->uid==stat.st_uid))
-				rest=fs->chattr(din,inode,type,attr,stat.st_uid,stat.st_gid,stat.st_atime,stat.st_mtime);
+				rest=fs->chattr(0,inode,attr,stat.st_uid,stat.st_gid,stat.st_atime,stat.st_mtime);
 			else rest=-EPERM;
 			break;
 		case CHATTR_UID:
 			if(proc->uid==0)
-				rest=fs->chattr(din,inode,type,stat.st_mode,attr,stat.st_gid,stat.st_atime,stat.st_mtime);
+				rest=fs->chattr(0,inode,stat.st_mode,attr,stat.st_gid,stat.st_atime,stat.st_mtime);
 			else rest=-EPERM;
 			break;
 		case CHATTR_GID:
 			if(((proc->uid==0)||(proc->uid==stat.st_uid))&&(attr==proc->gid))
-				rest=fs->chattr(din,inode,type,stat.st_mode,stat.st_gid,attr,stat.st_atime,stat.st_mtime);
+				rest=fs->chattr(0,inode,stat.st_mode,stat.st_gid,attr,stat.st_atime,stat.st_mtime);
 			else rest=-EPERM;
 			break;
 		case CHATTR_UTIME:
@@ -1996,10 +1858,10 @@ static int chattr(FS *fs,int din,uint inode,int type,int cmd,uint attr)
 				if(ut==NULL)
 				{
 					time=sys_time(NULL);
-					rest=fs->chattr(din,inode,type,stat.st_mode,stat.st_gid,attr,time,time);
+					rest=fs->chattr(0,inode,stat.st_mode,stat.st_gid,attr,time,time);
 				}
 				else
-					rest=fs->chattr(din,inode,type,stat.st_mode,stat.st_gid,attr,ut->actime,ut->modtime);
+					rest=fs->chattr(0,inode,stat.st_mode,stat.st_gid,attr,ut->actime,ut->modtime);
 			}
 			else rest=-EPERM;
 			break;
@@ -2011,11 +1873,10 @@ static int chattr(FS *fs,int din,uint inode,int type,int cmd,uint attr)
 
 int sys_chattr(const char *path,int cmd,uint attr)
 {
-	uint inode=0,block;
+	uint inode=0;
 	int rest;
 	int open=0;
 	int din;
-	int type;
 	SEARCH_PARAM start,end;
 	FS *fs;
 
@@ -2030,25 +1891,18 @@ int sys_chattr(const char *path,int cmd,uint attr)
 		fs=end.vent->fs;
 		din=end.vent->din;
 		inode=end.vent->inode;
-		type=end.vent->type;
 	}
 	else
 	{
 		fs=start.vent->fs;
 		din=start.vent->din;
-		type=NORMAL_FILE;
-		if((rest=fs->open(start.path,start.vent->din,start.vent->inode,0,&inode))==-EISDIR)
-		{
-			rest=fs->opendir(start.path,start.vent->din,start.vent->inode,&inode,&block);
-			type=DIRECTORY;
-		}
-		if(rest<0)goto ERR;
+		if((rest=fs->open(start.path,start.vent->din,start.vent->inode,0,&inode))<0)goto ERR;
 		open=1;
 	}
 
-	rest=chattr(fs,din,inode,type,cmd,attr);
-	if(open)fs->close(start.vent->din,inode,type);
+	rest=chattr(fs,din,inode,cmd,attr);
 ERR:
+	if(open)fs->close(start.vent->din,inode);
 	releaseVentObj(end.vent);
 
 	return rest;
@@ -2062,10 +1916,9 @@ int sys_fchattr(int fd_num,int cmd,uint attr)
 
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
-	if(fd->vent->type!=NORMAL_FILE)return -EBADF;
 
 	setVentObj(fd->vent);
-	rest=chattr(fd->vent->fs,fd->vent->din,fd->vent->inode,NORMAL_FILE,cmd,attr);
+	rest=chattr(fd->vent->fs,fd->vent->din,fd->vent->inode,cmd,attr);
 	releaseVentObj(fd->vent);
 
 	return rest;
@@ -2150,229 +2003,6 @@ END:
 	return rest;
 }
 
-
-/*
- * PRIVATE
- * parameters : number of fds,fd_set,fdե饰polling flag
- * return : 0ʾ=Ѳfd number or -1=Ѳʤ or -2=fdʤ or error number
- */
-static int sendFdset(int nfds,fd_set *fdset,int flag,int polling)
-{
-	char *fdbit;
-	F_DSC *fd;
-	int rest = -2;
-	int i,j,k;
-
-
-	fdbit = (char*)fdset;
-	for (i = 0; i < sizeof(fd_set); ++i)
-		if (fdbit[i] != 0)
-			for (j = 0; j < BYTE_BIT; ++j)
-				if (fdbit[i] & (1 << j))
-				{
-					if ((k = i * BYTE_BIT + j) >= nfds)
-						return rest;
-
-					if ((fd = getFd(k)) == NULL)
-						return -EBADF;
-					rest = fd->vent->fs->select(fd->vent->inode,flag,polling);
-					if (rest == 1)
-						return k;
-					if (rest < 0)
-						return rest;
-					rest = -1;
-				}
-
-	return rest;
-}
-
-static void inline clearFdset(fd_set **fdset,int cnt)
-{
-	int i;
-
-
-	for(i=0;i<cnt;++i)
-		fdset[i]->bitmap=0;
-}
-
-/* seconds=-1ʥȯޤǥ֥å롣 */
-int sys_select(int nfds,fd_set *readfds,fd_set *writefds,fd_set *errorfds,int seconds)
-{
-	int polling = 0;
-	int rest;
-	int flag[3];
-	fd_set *fdset[3];
-	int i,cnt,cnt2;
-
-
-	cnt = 0;
-	if (readfds != NULL)
-	{
-		if(checkMem(readfds) == -1)
-			return -EFAULT;
-		flag[cnt] = FD_READ;
-		fdset[cnt++] = readfds;
-	}
-	if (writefds != NULL)
-	{
-		if (checkMem(writefds) == -1)
-			return -EFAULT;
-		flag[cnt] = FD_WRITE;
-		fdset[cnt++] = writefds;
-	}
-	if (errorfds != NULL)
-	{
-		if (checkMem(errorfds) == -1)
-			return -EFAULT;
-		flag[cnt] = FD_ERR;
-		fdset[cnt++] = errorfds;
-	}
-
-	/* ʥԤ */
-	if (cnt == 0)
-	{
-		if (seconds > 1)
-			sys_sleep(seconds);
-		else if (seconds == -1)
-			sigWait();
-		return 0;
-	}
-
-	if(seconds == 0)
-		polling=1;
-	cnt2 = 0;
-	for(;;)
-	{
-		for (i = 0; i < cnt; ++i)
-		{
-			rest = sendFdset(nfds,fdset[i],flag[i],polling);
-			if (rest >= 0)										/* Ѳꡣ*/
-			{
-				clearFdset(fdset,cnt);
-				((char*)fdset[i])[rest / BYTE_BIT] |= 1 << (rest % BYTE_BIT);
-
-				return 1;
-			}
-			if (rest < -2)
-				return rest;							/* 顼*/
-			if (rest == -2)
-				cnt2 += 1;
-		}
-
-		if ((polling == 1) || (cnt2 == cnt))						/* polling or no fd. */
-		{
-			clearFdset(fdset,cnt);
-			return 0;
-		}
-		if (seconds == -1)
-			sigWait();
-		else
-			sys_sleep(seconds);
-
-		polling=1;
-	}
-}
-
-
-int sys_poll(struct pollfd *ufds, uint nfds, int seconds)
-{
-	int polling,flag;
-	int rest;
-	int i;
-	F_DSC *fd;
-
-
-	if (checkMem(ufds) == -1)
-		return -EFAULT;
-
-	polling = (seconds == 0)? 1 : 0;
-
-	/* Init revents. */
-	for (i = 0; i < nfds; ++i)
-		ufds[i].revents = 0;
-
-	for (;;)
-	{
-		for (i = 0; i < nfds; ++i)
-		{
-			/* Get file descriptor. */
-			if (( fd = getFd(ufds[i].fd)) == NULL)
-			{
-				ufds[i].revents = POLLNVAL;
-				return 1;
-			}
-
-			/* Get flag. */
-			switch(ufds[i].events)
-			{
-				case POLLIN:
-					flag = FD_READ;
-					break;
-				case POLLOUT:
-					flag = FD_WRITE;
-					break;
-				case POLLERR:
-				case POLLNVAL:
-				case POLLHUP:
-					flag = FD_ERR;
-					break;
-				default:
-					return -EINVAL;
-			}
-
-			/* Send poll. */
-			rest = fd->vent->fs->select(fd->vent->inode,flag,polling);
-			if (rest == 1)
-			{
-				ufds[i].revents=ufds[i].events;
-				return 1;
-			}
-			else if (rest < 0)
-			{
-				ufds[i].revents = POLLERR;
-				return 1;
-			}
-		}
-
-		if (polling == 1)
-			break;
-
-		/* Wait. */
-		if (seconds < 0)
-			sigWait();
-		else
-			sys_sleep(seconds / 1000);
-
-		polling = 1;
-	}
-
-	return 0;
-}
-
-
-int sys_getfsstat(struct statfs *buf,long bufsize,int flags)
-{
-	int cnt;
-
-
-	if(buf==NULL)cnt=INT_MAX;
-	else cnt=bufsize/sizeof(struct statfs);
-	
-	return searchMount(buf,cnt);
-}
-
-
-int sys_fstatvfs(int fildes, struct statfs * buf)
-{
-	F_DSC *fd;
-
-
-	if(checkMem(buf)==-1)return -EFAULT;
-	if((fd=getFd(fildes))==NULL)return -EBADF;
-
-	return getFsstat(fd->vent,buf);
-}
-
 /************************************* ̤ ***************************************/
 int sys_lock()
 {
@@ -2455,25 +2085,22 @@ int mountRoot(const char *dev_path,const char *fs_name)
 
 
 	/* Search file system number */
-	if ((fs = search_fs(fs_name)) == NULL)
-		return -ENODEV;
+	if((fs=search_fs(fs_name))==NULL)return -ENODEV;
 
 	/* ǥХץ */
-	if ((mount_din = mountDevice(dev_path)) < 0)
-		return mount_din;
+	if((mount_din=mountDevice(dev_path))<0)return mount_din;
 
 	/* ޥȴؿƤӽФ */
-	if ((root_blk = fs->mount(mount_din)) == -1)
-		goto ERR;
+	if((root_blk=fs->mount(mount_din))==-1)goto ERR;
 
 	/* 롼Ȳۥǥ쥯ȥꡣ */
-	rootEnt->din = mount_din;
-	rootEnt->fs = fs;
-	rootEnt->inode = root_blk;
+	rootEnt->din=mount_din;
+	rootEnt->fs=fs;
+	rootEnt->inode=root_blk;
 
 	return 0;
 ERR:
-	fs->close(0,mount_din,0);
+	fs->close(0,mount_din);
 
 	return -1;
 }
@@ -2515,14 +2142,8 @@ int initProc1Fs()
 }
 
 /*************************************************************************************************/
-void test_fs(PROC *proc)
+void test_fs()
 {
-	static FILE_STRUCT *fst;
-	F_DSC *fd ;
-	
-	if (proc !=NULL)
-		fst = proc->file_struct;
-	fd = fst->fd[0];
-	printk("fd=%x fd->vent=%x\n",fd,fd->vent);
+	printk("block=%x\n",((VENTRY*)0x16cf80)->inode);
 }
 /**************************************************************************************************/
