@@ -38,20 +38,14 @@ static int write(void* buf,size_t size,size_t begin)
 }
 
 
-static int ioctl(int cmd,...)
+static int ioctl(int prm_num,int cmd,...)
 {
 	return 0;
 }
 
 
-static int select(int flag,int pollong)
-{
-	return 1;
-}
-
-
 static DEV_INFO devInfo={
-	"null",0,0,0,open,read,write,ioctl,select
+	"null",0,0,0,open,read,write,ioctl,
 };
 
 
