@@ -33,7 +33,6 @@
 #include"device.h"
 #include"signal.h"
 #include"mm.h"
-#include"debug.h"
 #include"test.h"
 
 
@@ -512,46 +511,39 @@ static uint findAllMemsize()
  *
  ***********************************************************************************/
 
-#define HEAP_NUM() (sizeof(heapHead)/sizeof(HEAP_HEAD))
-
-/* ʬ꡼إå */
-typedef struct MEM_HEAD{
-	struct MEM_HEAD *next;	/* ζ꡼Ƭ */
-	int size;				/* ƥ */
-}MEM_HEAD;
+typedef struct HEAP_MEM{
+	char* last;
+	struct HEAP_MEM *next;
+}HEAP_MEM;
 
 /*
- * ҡץإå
- * Note! : "memTop"ΰ֤ɬ"MEM_HEAD""next"Ʊˤ뤳
+ * Note!
+ *  "mem"HEAP_MEM"next"ƱեåȰ֤֤ȡ
  */
 typedef struct{
-	MEM_HEAD *memTop;		/* ꡼Ƭ */
-	uint size;				/* ƥ */
-	int gate;				/* ԥåѥ */
-}HEAP_HEAD;
+	int dummy;		/* HEAP_MEM"last"Ʊեåȡkfree()HEAP_MEMȸʤȤɬס */
+	HEAP_MEM *mem;
+	uint size;
+	int gate;
+}HEAP;
 
 
-/*
- * PRIVATE
- * ҡץإå
- */
-static HEAP_HEAD heapHead[]={
-	{NULL,0,     0},
-	{NULL,0x10,  0},
-	{NULL,0x20,  0},
-	{NULL,0x40,  0},
-	{NULL,0x80,  0},
-	{NULL,0x100, 0},
-	{NULL,0x200, 0},
-	{NULL,0x400, 0},
-	{NULL,0x800, 0},
-	{NULL,0x1000,0},
+/* PRIVATE kmallocѹ¤Ρ */
+static HEAP heap[]={
+	{0,NULL,0,     0},
+	{0,NULL,0x10,  0},
+	{0,NULL,0x20,  0},
+	{0,NULL,0x40,  0},
+	{0,NULL,0x80,  0},
+	{0,NULL,0x100, 0},
+	{0,NULL,0x200, 0},
+	{0,NULL,0x400, 0},
+	{0,NULL,0x800, 0},
+	{0,NULL,0x1000,0},
 };
 
-/*
- * PRIVATE
- * Хͤҡ׹¤ΥʥС
- */
+
+/* PRIVATE,Хͤҡ׹¤ΥʥС롣 */
 static uchar heapNum1[]={
 	0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
 	1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
@@ -570,12 +562,16 @@ static uchar heapNum1[]={
 	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
 	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
 };
+
 static uchar heapNum2[]={
 	0,5,6,7,7,8,8,8,8,9,9,9,9,9,9,9,
 	9,
 };
 
 
+#define HEAP_NUM() (sizeof(heap)/sizeof(HEAP))
+
+
 /*
  * PRIVATE
  * ҡץ롣
@@ -584,7 +580,7 @@ static uchar heapNum2[]={
  */
 static inline int getHeapSize(size_t size)
 {
-	if(size>=0x100)
+	if(size>0x100)
 		return heapNum2[ROUNDUP(size,0x100)>>8];
 	return heapNum1[size];
 }
@@ -593,65 +589,56 @@ static inline int getHeapSize(size_t size)
 /*
  * GLOBAL
  * 2Ѿ襵Ȥ˥ڡ꡼ʬ䤹
+ * ϥڡ
  * parameters : Size by byte
  * return : allocate address or NULL
  */
 void *kmalloc(size_t size)
 {
-	union{
-		MEM_HEAD *head;
-		uint ui;
-	}mem,alloc;
+	char *addr;
 	int i;
+	HEAP_MEM *mem;
 
 
-	if (size == 0)
-		return NULL;
-
-	if (size <= PAGE_SIZE)
+	if(size==0)return NULL;
+	else if(size<=PAGE_SIZE)
 	{
-		i = getHeapSize(size);
+		addr=NULL;
+		i=getHeapSize(size);
 
-		enter_spinlock(&heapHead[i].gate);
+		enter_spinlock(&heap[i].gate);
 		{
-			if (heapHead[i].memTop == NULL)
+			mem=heap[i].mem;
+
+			if(mem==NULL)
 			{
 				/* 1ڡ꡼ݡ */
-				mem.head = allocKernelPage();
-				if (mem.head == NULL)
-				{
-					exit_spinlock(&heapHead[i].gate);
-					return NULL;
-				}
+				if((mem=allocKernelPage())==NULL)goto OUT;
 
 				/*
 				 * Ȥaddress狼褦ˤ롣
 				 */
-				getPhysTableFromAddr(mem.head)->heap_num = i;
+				getPhysTableFromAddr(mem)->heap_num=i;
 
-				mem.head->next = NULL;
-				mem.head->size = PAGE_SIZE;
-				heapHead[i].memTop = mem.head;
+				heap[i].mem=mem;
+				mem->next=NULL;
+				mem->last=(char*)mem+PAGE_SIZE;
 			}
-
-			mem.head = heapHead[i].memTop;
-			mem.head->size -= heapHead[i].size;
-			if (mem.head->size == 0)
-				heapHead[i].memTop = mem.head->next;
+			addr=mem->last-heap[i].size;
+			if(addr==(char*)mem)heap[i].mem=mem->next;
+			else mem->last=addr;
 		}
-		exit_spinlock(&heapHead[i].gate);
-
-		alloc.ui = mem.ui + mem.head->size;
+OUT:
+		exit_spinlock(&heap[i].gate);
 	}
 	else
 	{
-		size = ROUNDUP(size,PAGE_SIZE);
-		if ((alloc.head = allocMultiKernelPage(size / PAGE_SIZE)) == NULL)
-			return NULL;
-		getPhysTableFromAddr(alloc.head)->heap_num = HEAP_NUM() + size / PAGE_SIZE;
+		size=ROUNDUP(size,PAGE_SIZE);
+		if((addr=allocMultiKernelPage(size/PAGE_SIZE))==NULL)return NULL;
+		getPhysTableFromAddr(addr)->heap_num=HEAP_NUM()+size/PAGE_SIZE;
 	}
 
-	return alloc.head;
+	return addr;
 }
 
 
@@ -662,104 +649,88 @@ void *kmalloc(size_t size)
  */
 void kfree(void *ptr)
 {
-	union{
-		MEM_HEAD *head;
-		uint ui;
-	}mem,free,prev,next;
-	uint i,m;
+	int i;
+	HEAP_MEM *mem,*prev;
+	uint m;
 
 
-	if (ptr == NULL)
-		return;
+	if(ptr==NULL)return;
 
-	if ((i = getPhysTableFromAddr(ptr)->heap_num) == 0)
-	{
+	if((i=getPhysTableFromAddr(ptr)->heap_num)==0)
 #ifdef DEBUG
-		printk("kfree() : 0x%x did not allocated\n",ptr);
-#endif
+	{
+		printk("kfree() error! free address=%x\n",ptr);
 		return;
 	}
-
-	if (i < HEAP_NUM())
+	if((uint)ptr&(heap[i].size-1))
 	{
-		free.head = ptr;
-#ifdef DEBUG
-		/* Υå */
-		mem.head = free.head;
-		mem.ui &= heapHead[i].size - 1;
-		if (mem.ui != 0)
-		{
-			printk("kfree() : 0x%x is no complete\n",free.head);
-			return;
-		}	
+		printk("kfree() error! illegal address=%x\n",ptr);
+		return;
+	}
+#else
+		return;
 #endif
-		enter_spinlock(&heapHead[i].gate);
+	if(i<HEAP_NUM())
+	{
+		mem=(HEAP_MEM*)ptr;
+		prev=(HEAP_MEM*)&heap[i];
+
+		enter_spinlock(&heap[i].gate);
 		{
-			/* ꡼³󥯰֤򸡺 */
-			mem.head = (MEM_HEAD*)&heapHead[i];
-			for (;mem.head->next != NULL; mem.head = mem.head->next)
-				if (free.head < mem.head->next)
-					break;
-			prev = mem;
+			/* Link */
+			for(;prev->next!=NULL;prev=prev->next)
+				if(prev->next>=mem)break;
 #ifdef DEBUG
-			if ((mem.head != (MEM_HEAD*)&heapHead[i]) && (free.ui < (mem.ui + mem.head->size)) )
+			if(prev->next==mem)
 			{
-				printk("kfree() : 0x%x is already free\n",free.head);
-				goto EXIT;
+				printk("kfree() error! 0x%x memory is already freed\n",ptr);
+				goto NEXT;
 			}
 #endif
-			/* Υ꡼ȷҤ */
-			next.head = prev.head->next;
-			if ((next.ui == (free.ui + heapHead[i].size)) && ((next.ui / PAGE_SIZE) == (free.ui / PAGE_SIZE)) )
-			{
-				free.head->next = next.head->next;
-				free.head->size = next.head->size + heapHead[i].size;
-			}
-			else
-			{
-				free.head->next = next.head;
-				free.head->size = heapHead[i].size;
-			}
+			mem->next=prev->next;
+			prev->next=mem;
+			mem->last=(char*)ptr+heap[i].size;
 
-			/* Ĥʤ꡼ڡˤʤäڡ */
-			if (free.head->size == PAGE_SIZE)
+			/* ɥ쥹³ƤХ꡼Ĥʤ */
+			if(mem->last==(char*)mem->next)
 			{
-				prev.head->next = free.head->next;
-				getPhysTableFromAddr(free.head)->heap_num = 0;
-				unlinkPhysTable(free.ui);
-				goto EXIT;
-			}
+				mem->last=mem->next->last;
+				mem->next=mem->next->next;
 
-			/* Υ꡼ȷҤ */
-			if (((prev.ui + prev.head->size) == free.ui) && ((prev.ui / PAGE_SIZE) == (free.ui / PAGE_SIZE)) )
-			{
-				prev.head->next = free.head->next;
-				prev.head->size += free.head->size;
-				
+				/* Ĥʤ꡼ڡˤʤäڡ */
+				if(!((uint)mem&(PAGE_SIZE-1))&&!((uint)mem->last&(PAGE_SIZE-1)))
+				{
+					prev->next=mem->next;
+					getPhysTableFromAddr(ptr)->heap_num=0;
+					unlinkPhysTable((uint)mem);
+					goto NEXT;
+				}
 			}
-			else
-				prev.head->next = free.head;
-
-			/* Ĥʤ꡼ڡˤʤäڡ */
-			if (prev.head->size == PAGE_SIZE)
+			if(prev->last==(char*)mem)
 			{
-				for (mem.head = (MEM_HEAD*)&heapHead[i]; mem.head->next != prev.head; mem.head = mem.head->next);
-				mem.head->next = free.head->next;
-				getPhysTableFromAddr(prev.head)->heap_num = 0;
-				unlinkPhysTable(prev.ui);
-				goto EXIT;
+				prev->last=mem->last;
+				prev->next=mem->next;
+
+				if(!((uint)prev&(PAGE_SIZE-1))&&!((uint)prev->last&(PAGE_SIZE-1)))
+				{
+					mem=prev;
+					prev=(HEAP_MEM*)&heap[i];
+					for(;prev->next!=mem;prev=prev->next);
+					prev->next=mem->next;
+					unlinkPhysTable((uint)mem);
+				}
 			}
 		}
-EXIT:	exit_spinlock(&heapHead[i].gate);
+NEXT:	exit_spinlock(&heap[i].gate);
 	}
 	else
 	{
-		getPhysTableFromAddr(ptr)->heap_num = 0;
-		m = (uint)ptr;
-		for (i -= HEAP_NUM(); i > 0; --i)
+		getPhysTableFromAddr(ptr)->heap_num=0;
+		m=(uint)ptr;
+		for(i-=HEAP_NUM();i>0;--i)
 		{
 			unlinkPhysTable(m);
-			m += PAGE_SIZE;
+			m+=PAGE_SIZE;
 		}
 	}
 }
@@ -825,7 +796,7 @@ static inline uint *getPageEntryAddr(uint addr,uint *ptbl)
  * parameters : ɥ쥹,ڡǥ쥯ȥꥢɥ쥹
  * return : ڡơ֥륨ȥ꡼ɥ쥹
  */
-/*static uint *getPageEntryAddrFromLinearAddr(uint addr,uint *page_dir)
+static uint *getPageEntryAddrFromLinearAddr(uint addr,uint *page_dir)
 {
 	uint *ptbl;
 
@@ -833,7 +804,7 @@ static inline uint *getPageEntryAddr(uint addr,uint *ptbl)
 	ptbl=(uint*)(*getPageDirEntryAddr(addr,page_dir)&~(PAGE_SIZE-1));
 	return getPageEntryAddr(addr,ptbl);
 }
-*/
+
 
 /*
  * PUBLIC
@@ -973,42 +944,32 @@ static inline void setBlockEmpty(uint pentry)
 
 
 /*
- * machine depend
  * PUBLIC
  * ڡХååץǥХλȲ1䤹
  * parameters : page entry
  */
 static inline void incBackupCount(uint *pentry)
 {
-	union{
-		uint ui;
-		uchar uc[4];
-	}value;
-
-
-	value.ui = *pentry;
-	++value.uc[3];
-	*pentry = value.ui;
+	rotateR(pentry,24);
+	++(uchar)*pentry;
+	rotateL(pentry,24);
 }
 
 
 /*
- * machine depend
  * PUBLIC
  * ڡХååץǥХλȲ1餹
  * parameters : page entry
  */
 static inline void decBackupCount(uint *pentry)
 {
-	union{
-		uint ui;
-		uchar uc[4];
-	}value;
-
-
-	value.ui = *pentry;
-	if (value.uc[3] == 0)
+	rotateR(pentry,24);
+	if((uchar)(--*pentry)==0)
+	{
+		rotateL(pentry,24);
 		setBlockEmpty(*pentry);
+	}
+	else rotateL(pentry,24);
 }
 
 
@@ -1026,23 +987,21 @@ static int backupPage(uint phaddr,PROC *proc,uint *pentry)
 	enter_spinlock(&((MM_STRUCT*)proc->mm_struct)->pageDirGate);
 	{
 		/* Ǥ˥꡼ϳƤ롣 */
-		if (((*pentry & PAGE_PRESEN) == 0) || (phaddr != (*pentry & ~(PAGE_SIZE - 1))) )
+		if(((*pentry&PAGE_PRESEN)==0)||(phaddr!=(*pentry&~(PAGE_SIZE-1))))
 			goto EXIT;
 
 		/* page tableԺߤ */
-		*pentry &= ~PAGE_PRESEN;
+		*pentry&=~PAGE_PRESEN;
 
 		/* ꤵץ¹ʤpage tableTBLեå夹 */
 		if((proc->cpu!=get_current_cpu())&&(cputask[proc->cpu].current_task==proc))
 			issue_interrupt_to_cpu(proc->cpu,FLASH_PAGEDIR_VECT);
 
 		/* Store page. */
-		if ((blk = getEmptyBackup()) == -1)
-			goto ERR;
-		if (phaddr < KERNEL_END)
+		if((blk=getEmptyBackup())==-1)goto ERR;
+		if(phaddr<KERNEL_END)
 		{
-			if (write_cache(pbDin,(void*)phaddr,pbSectors,blk) != pbSectors)
-				goto ERR;
+			if(write_cache(pbDin,(void*)phaddr,pbSectors,blk)!=pbSectors)goto ERR;
 		}
 		else
 		{
@@ -1092,8 +1051,7 @@ ERR:
 static int restorePage(void *buf,uint *pentry)
 {
 	/* ǥХɤ߹ߡ */
-	if (read_cache(pbDin,buf,pbSectors,getBlock(*pentry)) != pbSectors)
-		return -1;
+	if(read_cache(pbDin,buf,pbSectors,getBlock(*pentry))!=pbSectors)return -1;
 
 	/* ȥ󥿤򸺤餹 */
 	decBackupCount(pentry);
@@ -1114,11 +1072,10 @@ int initBackupDev(const char *dev_path)
 
 
 	/* Device open. */
-	if ((pbDin = mountPageBackupDev(dev_path)) < 0)
-		return pbDin;
+	if((pbDin=mountPageBackupDev(dev_path))<0)return pbDin;
 
 	/* ѡƥ󥿥פγǧ */
-	if(get_devstat(pbDin,&dev_stat)==-1)return -1;
+	if(get_dev_stat(pbDin,&dev_stat)==-1)return -1;
 	if(dev_stat.prt_type!=PT_TYPE_PAGING)return -1;
 	pbSectors=PAGE_SIZE/dev_stat.sect_size;
 
@@ -1314,7 +1271,7 @@ void *forkPage(MM_STRUCT *org_mm,void **stack)
 
 		/* 桼åγơ */
 		if((ptbl=(uint*)allocKernelPage())==NULL)goto ERR;
-		if((user_stack = allocUserPage(&ptbl[PAGE_SIZE/sizeof(uint)-1]))==NULL)goto ERR2;
+		if((user_stack=allocUserPage(&ptbl[PAGE_SIZE/sizeof(uint)-1]))==NULL)goto ERR2;
 		if(pdir[USER_STACK_BEG/PAGE_DIR_SIZE]!=0)
 		{
 			org_ptbl=(uint*)(pdir[USER_STACK_BEG/PAGE_DIR_SIZE]&~(PAGE_SIZE-1));
@@ -1388,7 +1345,7 @@ int loadBinary(OPEN_F *open_f,size_t size,size_t load_size,uint address,int attr
 
 			/* ꡼°ꤹ롣 */
 			pentry=getPageEntryAddr(addr,ptbl);
-			if ((page = allocUserPage(pentry))==NULL)goto ERR;
+			if((page=allocUserPage(pentry))==NULL)goto ERR;
 			*pentry=(uint)page|PAGE_PRESEN|PAGE_USER|PAGE_RW;
 		}
 	}
@@ -1531,21 +1488,6 @@ void releasePageDir(void *mm_struct)
  *  桼åڡԺߤνθľ(¤ߤ)
  * parameters : page fault address,Exception frame
  */
-static void printErr(uint errnum,uint addr,uint eip,uint cs,uint esp,uint ss)
-{
-	if(cs==KERNEL_CODE_DES)
-	{
-#ifdef DEBUG	
-		printSyscallEntry(5);
-#endif		
-		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x process=%x\n",
-			errnum,addr,eip,cs,get_current_task());
-	}		
-	else
-		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x\n",
-			errnum,addr,eip,cs,esp,ss);
-}
-
 void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 {
 	/* 顼ɥӥåȥޥ */
@@ -1564,22 +1506,15 @@ void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 	MM_STRUCT *mm_struct;
 
 
-	mm_struct = get_current_task()->mm_struct;
+	mm_struct=(MM_STRUCT*)get_current_task()->mm_struct;
 
 	/* øڡȿ */
-	if ((fault_addr < USER_BEG)   ||
-	    (IOMAP_BEG <= fault_addr) ||
-	    ((mm_struct->lastPageAddr < fault_addr) && (fault_addr < USER_STACK_BEG)) )
+	if((fault_addr<USER_BEG)||(fault_addr>=IOMAP_BEG)||((fault_addr>mm_struct->lastPageAddr)&&(fault_addr<USER_STACK_BEG)))
 	{
-		/* 桼ʥϥɥ餫ξ硣 */
-		if((fault_addr==(uint)returnUserHandler)&&(getBeforeEsp()))
-		{
-			get_current_task()->nest_count-=1;
-			returnUserHandler();
-		}	
-
 		printk("\nPage privilege violation!\n");
-		printErr(frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
+		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x,proc=%x\n",
+				frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss,get_current_task());
+
 		forceSendSignal(get_current_task(),SIGSEGV);
 /***************************************/
 idle();
@@ -1595,18 +1530,18 @@ idle();
 		/* ڡǥ쥯ȥγơ */
 		if(*dentry==0)
 		{
-			if ((*dentry = (uint)allocKernelPage()) == 0)
+			if((*dentry=(uint)allocKernelPage())==0)
 			{
 				printk("Fail alloc physical page!\n");
 				sys_exit(1);
 			}
-			memset ((void*)*dentry,0,PAGE_SIZE);
-			*dentry |= PAGE_PRESEN | PAGE_RW | PAGE_USER;
+			memset((void*)*dentry,0,PAGE_SIZE);
+			*dentry|=PAGE_PRESEN|PAGE_RW|PAGE_USER;
 		}
 
 		/* ʪڡγơ */
-		current_pentry = getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
-		if ((alloc_page = allocUserPage(current_pentry)) == NULL)
+		current_pentry=getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
+		if((alloc_page=allocUserPage(current_pentry))==NULL)
 		{
 			printk("Fail alloc physical page!\n ");
 			sys_exit(1);
@@ -1615,8 +1550,8 @@ idle();
 		enter_spinlock(&mm_struct->pageDirGate);
 		{
 			/* Хååפꡣ */
-			if (*current_pentry != 0)
-				if (restorePage(alloc_page,current_pentry) == -1)
+			if(*current_pentry!=0)
+				if(restorePage(alloc_page,current_pentry)==-1)
 				{
 					printk("Fail restore page!\n ");
 					exit_spinlock(&mm_struct->pageDirGate);
@@ -1625,12 +1560,15 @@ idle();
 					return;
 				}
 
-			*current_pentry = (uint)alloc_page | PAGE_PRESEN | PAGE_RW | PAGE_USER;
+			*current_pentry=(uint)alloc_page|PAGE_PRESEN|PAGE_RW|PAGE_USER;
 		}
 		exit_spinlock(&mm_struct->pageDirGate);
 	}
 	else	/* ڡȿ */
 	{
+		/* 桼ʥϥɥ餫ξ硣 */
+		if((fault_addr==(uint)returnUserHandler)&&(getBeforeEsp()))returnUserHandler();
+
 		current_pentry=getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
 
 		/* ԡ饤Ƚ */
@@ -1665,7 +1603,7 @@ idle();
 						new_entry=(uint*)(((uint)current_pentry&(PAGE_SIZE-1))|(uint)alloc_tbl);
 					}
 					else pdir[array]&=~PT_COPY_ON_WRITE;
-				}
+				} 
 
 				/* ʪڡơڡƤ򥳥ԡ롣 */
 				if((alloc_page=allocUserPage(new_entry))==NULL)
@@ -1681,7 +1619,13 @@ idle();
 		else	/* ɤ߹ߥڡȿ */
 		{
 			printk("\nRead only page violation!\n");
-			printErr(frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
+			if(frame.cs==KERNEL_CODE_DES)
+				printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x process=%x\n",
+					frame.error,fault_addr,frame.eip,frame.cs,get_current_task());
+			else
+				printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x\n",
+					frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
+
 			forceSendSignal(get_current_task(),SIGSEGV);
 
 			return;
@@ -1856,9 +1800,10 @@ void initMm()
 	initPhysMemTable();
 }
 /************************************************************************/
-int test_mm(uint *page)
+int test_mm()
 {
-	printk("40000c39=%x 40000080=%x\n",*(uint*)((uint)page + 0xc39),*(uint*)((uint)page + 0x80));
+	if(get_current_task()==(PROC*)0x150b00)
+		printk("ptbl entry=%x\n",*getPageEntryAddrFromLinearAddr(0x400091c4,getPageDir()));
 
 	return 0;
 }
