@@ -32,7 +32,7 @@ typedef struct{
 }TCP_HEADER;
 
 
-extern int releaseSckFromConect(SOCKET*);
+extern void releaseSckFromConect(SOCKET*);
 extern int receiveTcp(IP_HEADER*);
 extern int initTcp();
 
