@@ -279,7 +279,7 @@ PROC *searchProcList(int pid)
  *
  *******************************************************************************************************/
 
-extern int *active_cpu;		/* ư CPU */
+extern int active_cpu;		/* ư CPU */
 
 /* PUBLIC */
 static int procLockGate=0;		/* ¾Υץ¤λȻΥåȡ */
@@ -299,7 +299,7 @@ static int GetFewTaskCpu()
 
 
 	num=cputask[0].proc_num;
-	for(i = 1; i < *active_cpu; ++i)
+	for(i=1;i<active_cpu;++i)
 		if(cputask[i].proc_num<num)
 		{
 			num=cputask[i].proc_num;
@@ -435,10 +435,10 @@ static void releaseChildProc(PROC *proc)
  * PUBLIC
  * 1 exitҥץ롣
  * 2 λҥץξ֤𤹤롣
- * parameters : process structure,search pid,search pgid,task state,state address
+ * parameters : process structure,search pid,search gid,task state,state address
  * return : pid or 0 or error number
  */
-static int searchExitState(PROC *proc,pid_t pid,pid_t pgid,int state,int *stat_loc)
+static int searchExitState(PROC *proc,int pid,int gid,int state,int *stat_loc)
 {
 	int rest=-ECHILD;
 	PROC *p;
@@ -460,10 +460,10 @@ static int searchExitState(PROC *proc,pid_t pid,pid_t pgid,int state,int *stat_l
 				break;
 			}
 	}
-	else if(pgid!=0)
+	else if(gid!=0)
 	{
 		for(p=proc->child;p!=NULL;p=p->brother)
-			if(p->pgid==pgid)
+			if(p->gid==gid)
 			{
 				rest=0;
 				if(p->state&state)
@@ -539,10 +539,10 @@ void delNestCnt()
 /*
  * PRIVATE
  * ץ롼פθ
- * parameters : search start process,pgid
+ * parameters : search start process,gid
  * return : process or Failed=NULL
  */
-static PROC *searchPg(PROC *proc,pid_t pgid)
+static PROC *searchPg(PROC *proc,int gid)
 {
 	PROC *rest;
 
@@ -551,9 +551,9 @@ static PROC *searchPg(PROC *proc,pid_t pgid)
 	{
 		if(proc->state!=TASK_EXIT)
 		{
-			if(proc->pgid==pgid)return proc;
+			if(proc->gid==gid)return proc;
 			if(proc->child!=NULL)
-				if((rest=searchPg(proc->child,pgid))!=NULL)return rest;
+				if((rest=searchPg(proc->child,gid))!=NULL)return rest;
 		}
 	}while((proc=proc->brother)!=NULL);
 
@@ -582,20 +582,20 @@ static inline int getPid(PROC *proc)
  * parameters : search start process or NULL=proc1,groupe ID
  * return : prosecc or Failed=NULL
  */
-PROC *getNextProc(PROC *proc,pid_t pgid)
+PROC *getNextProc(PROC *proc,int gid)
 {
 	/* proc1õ */
 	if(proc==NULL)proc=proc1;
 
 	/* μν֤Υץõ */
-	if(proc->child!=NULL)return searchPg(proc->child,pgid);
+	if(proc->child!=NULL)return searchPg(proc->child,gid);
 	else
 	{
 		do
 		{
-			if(proc->brother!=NULL)return searchPg(proc->brother,pgid);
+			if(proc->brother!=NULL)return searchPg(proc->brother,gid);
 			if((proc=proc->parent)==NULL)break;
-		}while(proc->pgid==pgid);
+		}while(proc->gid==gid);
 	}
 
 	return NULL;
@@ -726,13 +726,11 @@ int sys_fork()
 	PROC *parent,*child;
 
 
-	parent = get_current_task();
-	if (parent->count_child >= MAX_CHILD)
-		return -EAGAIN;
+	parent=get_current_task();
+	if(parent->count_child>=MAX_CHILD)return -EAGAIN;
 
 	/* PROC¤ΤƤ */
-	if ((child = kmalloc(sizeof(PROC))) == NULL)
-		return -ENOMEM;
+	if((child=(PROC*)kmalloc(sizeof(PROC)))==NULL)return -ENOMEM;
 
 	/* 0 */
 	memset(&child->PROC_ZERO_TOP,0,(uint)&child->PROC_ZERO_END-(uint)&child->PROC_ZERO_TOP);
@@ -743,9 +741,8 @@ int sys_fork()
 
 	/* ID. */
 	child->pid=getPid(child);
-	child->pgid=parent->pgid;
-	child->uid=0;
 	child->gid=parent->gid;
+	child->uid=0;
 
 	/* å󥢥ɥ쥹(ü¤Υɥ쥹)򥳥ԡ */
 	child->ctlterm=parent->ctlterm;
@@ -760,12 +757,10 @@ int sys_fork()
 	if((child->signal_struct=setSignal(parent->signal_struct))==NULL)goto ERR;
 
 	/* ޡν */
-	setTimerStruct(child);
+	if((child->timer_struct=setTimerStruct())==NULL)goto ERR;
 
 	/* ҥץstack˥ƥȤꤹ */
-	esp = setContext((uint)stack);
-	if (esp == 0)
-		return 0;		/* ҥץ */
+	if((esp=setContext((uint)stack))==0)return 0;		/* ҥץ */
 
 	child->esp=esp;
 
@@ -787,7 +782,7 @@ ERR:
 
 int sys_exec(const char *path,char **argv,char **envp,volatile SYSCALL4_FRAME frame)
 {
-	enum{PARAM=sizeof(frame.para)};	/* ƥॳΰΥ */
+	enum{PARAM=sizeof(int)*4};	/* ƥॳΰο  Хȿ */
 
 	uint entry_addr,stack_esp;
 	int rest;
@@ -821,7 +816,7 @@ int sys_exec(const char *path,char **argv,char **envp,volatile SYSCALL4_FRAME fr
 	frame.ds=USER_DATA_DES;
 	frame.eip=entry_addr;
 	frame.cs=USER_CODE_DES;
-	frame.user_esp=stack_esp-PARAM;
+	frame.esp=stack_esp-PARAM;
 	frame.ss=USER_DATA_DES;
 
 	return 0;
@@ -865,7 +860,7 @@ void exit(int state,int signum)
 	releaseFileStruct(proc->file_struct);
 
 	/* ޡγ */
-	releaseTimer(&proc->timer,&proc->alarm);
+	releaseTimer(proc->timer_struct);
 
 	/* ʥγ */
 	releaseSignal(proc->signal_struct);
@@ -913,7 +908,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 
 	/* ǽλҥץ */
 	if(pid>0)rest=searchExitState(proc,pid,0,state,stat_loc);
-	else if(pid==0)rest=searchExitState(proc,0,proc->pgid,state,stat_loc);
+	else if(pid==0)rest=searchExitState(proc,0,proc->gid,state,stat_loc);
 	else if(pid==-1)rest=searchExitState(proc,0,0,state,stat_loc);
 	else rest=searchExitState(proc,0,-pid,state,stat_loc);
 
@@ -926,7 +921,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 
 	/* ҥץ */
 	if(pid>0)rest=searchExitState(proc,pid,0,state,stat_loc);
-	else if(pid==0)rest=searchExitState(proc,0,proc->pgid,state,stat_loc);
+	else if(pid==0)rest=searchExitState(proc,0,proc->gid,state,stat_loc);
 	else if(pid==-1)rest=searchExitState(proc,0,0,state,stat_loc);
 	else rest=searchExitState(proc,0,-pid,state,stat_loc);
 
@@ -937,7 +932,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 /*
  * Get process ID.
  */
-int sys_get_procid(pid_t pid,PROC_ID *pd)
+int sys_get_procid(int pid,PROC_ID *pd)
 {
 	PROC *proc;
 
@@ -951,10 +946,9 @@ int sys_get_procid(pid_t pid,PROC_ID *pd)
 	}
 
 	pd->pid=proc->pid;
-	pd->pgid=proc->pgid;
-	pd->ppid=proc->parent->pid;
-	pd->uid=proc->uid;
 	pd->gid=proc->gid;
+	pd->uid=proc->uid;
+	pd->ppid=proc->parent->pid;
 
 	return 0;
 }
@@ -963,7 +957,7 @@ int sys_get_procid(pid_t pid,PROC_ID *pd)
 /*
  * Set process ID.
  */
-int sys_set_procid(pid_t pid,PROC_ID *pd)
+int sys_set_procid(int pid,PROC_ID *pd)
 {
 	PROC *proc;
 
@@ -975,26 +969,20 @@ int sys_set_procid(pid_t pid,PROC_ID *pd)
 		if((proc=searchProcList(pid))==NULL)return -ESRCH;
 	}
 
-	if(pd->pgid>=0)
+	if(pd->gid>=0)
 	{
 		/* ץå꡼ */
 		if(isCtlproc(proc))return -EPERM;
 
 		/* ȥץλҥץǤʡ */
-		if(proc->pgid!=get_current_task()->pgid)return -EPERM;
+		if(proc->gid!=get_current_task()->gid)return -EPERM;
 
-		if(pd->pgid==0)proc->pgid=proc->pid;
-		else proc->pgid=pd->pgid;
+		if(pd->gid==0)proc->gid=proc->pid;
+		else proc->gid=pd->gid;
 	}
 	else if(pd->uid>=0)
 	{
-		if(proc->uid==0)proc->uid=pd->uid;
-		else return -EPERM;
-	}
-	else if(pd->gid>=0)
-	{
-		if(proc->uid==0)proc->gid=pd->gid;
-		else return -EPERM;
+		proc->uid=pd->uid;
 	}
 
 	return 0;
@@ -1079,9 +1067,7 @@ void set_tss(int des)
 int set_idle_proc(int cpu)
 {
 	/* Set process table */
-	idle_proc[cpu] = kmalloc(sizeof(PROC));
-	if (idle_proc[cpu] == NULL)
-		return -ENOMEM;
+	if((idle_proc[cpu]=(PROC*)kmalloc(sizeof(PROC)))==NULL)return -ENOMEM;
 	memset(idle_proc[cpu],0,sizeof(PROC));
 	idle_proc[cpu]->next=idle_proc[cpu]->prev=idle_proc[cpu];
 	idle_proc[cpu]->cpu=cpu;
