@@ -12,7 +12,7 @@
 
 #include<proc.h>
 #include<share/stropts.h>
-#include<sys/termios.h>
+#include<share/termios.h>
 
 
 #ifndef ASM_FILE
