@@ -8,16 +8,24 @@
 
 
 #include<sys/types.h>
-#include<share/dirent.h>
+#include<limits.h>
 
 
+typedef int DIR;
+
 struct dirent{
-	char d_name[255];	/* Name of entry. */
+	int  d_namlen;
+	char d_name[NAME_MAX];	/* Name of entry. */
 };
 
 
+extern inline int dirfd(DIR *dir)
+{
+	return (int)dir;
+}
+
 /* Functions. */
-extern int            closedir(DIR *);
+extern int            closedir(DIR *);;
 extern DIR           *opendir(const char *);
 extern struct dirent *readdir(DIR *);
 extern void           rewinddir(DIR *);
