@@ -10,18 +10,7 @@
 #define share_config_h
 
 
-#define MAX_CPU 8					/* бcpu */
-
-
-#ifndef ASM_FILE
-
-
-enum{
-	MAX_FILE_OPEN=20,				/* ץκե륪ץ */
-};
-
-
-#endif
+#define MAX_CPU 8								/* 対応最大cpu数 */
 
 
 #endif
