@@ -7,7 +7,7 @@
 #define share_stat_h
 
 
-#include<sys/types.h>
+#include<share/types.h>
 #include<share/stdint.h>
 #include<share/time.h>
 
@@ -31,7 +31,6 @@ struct stat{
 	uint32_t  st_blksize;	/* optimal blocksize for I/O */
 	dev_t	  st_rdev;		/* device type */
 	uint32_t  st_flags;		/* user defined flags for file */
-	int64_t	  st_blocks;	/* blocks allocated for file */
 	/* POSIX4. */
 	struct	timespec st_atimespec;	/* time of last access */
 	struct	timespec st_mtimespec;	/* time of last data modification */
