@@ -18,15 +18,10 @@ enum{
 	IOCTL_COM_TCDRAIN,
 	IOCTL_COM_TCFLASH,
 	IOCTL_COM_TCFLOW,
-
+	
 	TIOCSCTTY,			/* üꡣ */
 	TIOSPGRP,			/* üץ롼פѹ */
 	TIOGPGRP,			/* üץ롼פμ */
-	TIOCGWINSZ,			/* ̥μ */
-
-	SIOCATMARK,			/* Socket,at oob mark? */
-
-	I_TRUNC,			/* ե륵ڤͤᡢftruncate()  : */
 };
 
 
