@@ -25,9 +25,12 @@ typedef unsigned int   size_t;		/* Used for sizes of objects. */
 typedef int            ssize_t;		/* Used for a count of bytes or an error indication. */
 typedef int            time_t;		/* Used for time in seconds. */
 typedef unsigned int   uid_t;		/* Used for user IDs. */
-typedef unsigned int   clock_t;		/*  */
+typedef unsigned int   clock_t;		/* Used for system times in clock ticks or CLOCKS_PER_SEC. */
+typedef unsigned int   suseconds_t;	/* Used for time in microseconds. */
 
 /* ץ */
+typedef unsigned short	u_short;
+typedef unsigned int	u_int;
 typedef unsigned char	uchar;
 typedef unsigned short	ushort;
 typedef unsigned int	uint;
