@@ -8,16 +8,7 @@
 
 
 #include<sys/types.h>
-#include<sys/signal.h>
-
-
-enum{
-	NSIG=32,
-};
-
-
-extern const char *const sys_signame[NSIG];
-extern const char *const sys_siglist[NSIG];
+#include<share/signal.h>
 
 
 typedef int sig_atomic_t;
@@ -37,7 +28,6 @@ extern int	sigsuspend(const sigset_t *);
 extern int	sigsetpending(sigset_t *);
 extern int raise(int sig);
 extern sig_t signal(int, sig_t);
-extern int killpg(pid_t pgid, int sig);
 
 
 #endif
