@@ -1288,7 +1288,7 @@ int fclose(FILE *stream)
 	free(stream);
 
 	if(rest<0)return EOF;
-	else return 0;
+	return 0;
 }
 
 
@@ -1485,26 +1485,22 @@ char *fgets(char *s, int size, FILE *stream)
 
 	if(size<=0)return s;
 
-	if((rest=read(stream->fd,s,size-1))<0)
+	if((rest=read(stream->fd,s,size-1))<=0)
 	{
 		stream->flag|=F_FLAG_ERR;
 		return NULL;
 	}
 	s[rest]='\0';
-	for(i=0;s[i]!='\n';++i)
-		if(s[i]=='\0')
+
+	for(i=0;s[i]!='\0';++i)
+		if(s[i]=='\n')
 		{
-			if(rest<size-1)
-			{
-				stream->flag|=F_FLAG_EOF;
-				return NULL;
-			}
-			else return s;
+			s[++i]='\0';	/* ԤμNULL */
+			break;
 		}
-	s[i+1]='\0';
 
 	/* ե륪եåȤԤμޤᤷƤ */
-	lseek(stream->fd,rest-size+1,SEEK_CUR);
+	lseek(stream->fd,i-rest,SEEK_CUR);
 
 	return s;
 }
