@@ -11,6 +11,7 @@
 #include<share/syscall.h>
 #include<errno.h>
 #include<stdarg.h>
+#include<stdio.h>
 #include<sys/stat.h>
 #include<fcntl.h>
 
@@ -22,17 +23,12 @@ int open(const char *path,int oflag,...)
 
 
 	if(*path=='\0')return _error(ENOENT);
-
 	va_start(ap,oflag);
 	fd=syscall4(SYS_OPEN,(int)path,oflag,va_arg(ap,int)&~openMask);
 	va_end(ap);
 
-	if(fd<0)
-	{
-		errno=-fd;
-		return -1;
-	}
-	else return fd;
+	if(fd<0)return _error(-fd);
+	return fd;
 }
 
 
