@@ -18,17 +18,27 @@
 int openMask=0;
 
 
+/*
+ * timespec¤ΤؤβбΤν
+ * stat¤ΥСtimespec¤Τͤꤹ롣
+ */
+static void setTimespec(struct stat *buf)
+{
+	buf->st_atimespec.ts_sec=buf->st_atime;
+	buf->st_mtimespec.ts_sec=buf->st_mtime;
+	buf->st_ctimespec.ts_sec=buf->st_ctime;
+	buf->st_atimespec.ts_nsec=buf->st_mtimespec.ts_nsec=buf->st_ctimespec.ts_nsec=0;
+}
+
+
 int mkdir(const char *path, mode_t mode)
 {
 	int rest;
 
 
 	if((rest=syscall3(SYS_MKDIR,(int)path,(int)mode))<0)
-	{
-		errno=-rest;
-		return -1;
-	}
-	else return 0;
+		return _error(-rest);
+	return 0;
 }
 
 
@@ -38,11 +48,10 @@ int fstat(int fildes,struct stat *buf)
 
 
 	if((rest=syscall4(SYS_STAT,fildes,(int)NULL,(int)buf))<0)
-	{
-		errno=-rest;
-		return -1;
-	}
-	else return 0;
+		return _error(-rest);
+	setTimespec(buf);
+
+	return 0;
 }
 
 
@@ -52,11 +61,10 @@ int stat(const char *path,struct stat *buf)
 
 
 	if((rest=syscall4(SYS_STAT,0,(int)path,(int)buf))<0)
-	{
-		errno=-rest;
-		return -1;
-	}
-	else return 0;
+		return _error(-rest);
+	setTimespec(buf);
+
+	return 0;
 }
 
 
@@ -66,11 +74,19 @@ int chmod(const char *path, mode_t mode)
 
 
 	if((rest=syscall4(SYS_CHATTR,(int)path,CHATTR_MODE,(int)mode))<0)
-	{
-		errno=-rest;
-		return -1;
-	}
-	else return 0;
+		return _error(-rest);
+	return 0;
+}
+
+
+int fchmod(int fildes, mode_t mode)
+{
+	int rest;
+
+
+	if((rest=syscall4(SYS_FCHATTR,fildes,CHATTR_MODE,(int)mode))<0)
+		return _error(-rest);
+	return 0;
 }
 
 
@@ -96,3 +112,24 @@ int mkfifo(const char *pathname, mode_t mode)
 
 	return 0;
 }
+
+
+/********************** ̤ݡ **************************/
+
+
+int lstat(const char *path, struct stat *buf)
+{
+	return _error(ENOTSUP);
+}
+
+
+int mknod(const char *path, mode_t mode, dev_t dev)
+{
+	return _error(ENOTSUP);
+}
+
+
+int lchmod(const char *path, mode_t mode)
+{
+	return _error(ENOTSUP);
+}
