@@ -23,12 +23,15 @@ int ioctl(int fildes,int request,...)
 	switch(request)
 	{
 		case IOCTL_COM_TCDRAIN:
+		case TIOCSCTTY:
+		case TIOGPGRP:
 			rest=syscall4(SYS_IOCTL,fildes,0,request);
 			break;
 		case IOCTL_COM_TCGETTCATTR:
 		case IOCTL_COM_TCSENDBREAK:
 		case IOCTL_COM_TCFLASH:
 		case IOCTL_COM_TCFLOW:
+		case TIOSPGRP:
 			rest=syscall5(SYS_IOCTL,fildes,1,request,*arg);
 			break;
 		case IOCTL_COM_TCSETTCATTR:
@@ -43,5 +46,5 @@ int ioctl(int fildes,int request,...)
 		errno=-rest;
 		return -1;
 	}
-	else return 0;
+	else return rest;
 }
