@@ -312,10 +312,19 @@ int chdir(const char *path)
 
 
 	if((rest=syscall2(SYS_CHDIR,(int)path))<0)
-	{
-		errno=-rest;
-		return -1;
-	}
+		return _error(-rest);
+
+	return 0;
+}
+
+
+int fchdir(int fildes)
+{
+	int rest;
+
+
+	if((rest=syscall2(SYS_FCHDIR,fildes))<0)
+		return _error(-rest);
 
 	return 0;
 }
@@ -435,13 +444,6 @@ int close(int fildes)
 	int rest;
 
 
-	/* ɸϤϥʤ */
-	if((fildes>=STDIN_FILENO)&&(fildes<=STDERR_FILENO))
-	{
-		errno=EBADF;
-		return -1;
-	}
-
 	if((rest=syscall2(SYS_CLOSE,fildes))<0)
 	{
 		errno=-rest;
@@ -515,11 +517,25 @@ int chown(const char *path, uid_t owner, gid_t group)
 	if((int)group!=-1)
 		rest=syscall4(SYS_CHATTR,(int)path,CHATTR_GID,(int)group);
 	if((rest)<0)
-	{
-		errno=-rest;
-		return -1;
-	}
-	else return 0;
+		return _error(-rest);
+
+	return 0;
+}
+
+
+int fchown(int fildes, uid_t owner, gid_t group)
+{
+	int rest=0;
+
+
+	if((int)owner!=-1)
+		rest=syscall4(SYS_FCHATTR,fildes,CHATTR_UID,(int)owner);
+	if((int)group!=-1)
+		rest=syscall4(SYS_FCHATTR,fildes,CHATTR_GID,(int)group);
+	if((rest)<0)
+		return _error(-rest);
+
+	return 0;
 }
 
 
@@ -813,6 +829,24 @@ pid_t tcgetpgrp(int fd)
 
 char *getlogin(void)
 {
-	errno=ENOSYS;
+	errno=ENOTSUP;
 	return NULL;
 }
+
+
+int symlink(const char *path1, const char *path2)
+{
+	return _error(ENOTSUP);
+}
+
+
+ssize_t readlink(const char *path, char *buf, size_t bufsize)
+{
+	return _error(ENOTSUP);
+}
+
+
+int lchown(const char *path, uid_t owner, gid_t group)
+{
+	return _error(ENOTSUP);
+}
