@@ -11,6 +11,7 @@
 #include<share/syscall.h>
 #include<errno.h>
 #include<unistd.h>
+#include<sys/time.h>
 #include<utime.h>
 
 
@@ -26,3 +27,21 @@ int utime(const char *filename, struct utimbuf *buf)
 	}
 	else return 0;
 }
+
+
+int utimes(const char *path, const struct timeval *times)
+{
+	struct utimbuf buf;
+
+
+	buf.actime=times[0].tv_sec;
+	buf.modtime=times[1].tv_sec;
+
+	return utime(path,&buf);
+}
+
+
+int lutimes(const char *path, const struct timeval *times)
+{
+	return utimes(path,times);
+}
