@@ -32,6 +32,28 @@ enum{
 };
 
 
+/*****************************************************************************************
+ *
+ * Error handler.
+ *
+ ******************************************************************************************/
+
+
+/*
+ * PUBLIC
+ * 顼ɽ
+ */
+static void errstr(const char *mes)
+{
+	printf("%s\n",mes);
+}
+
+static void syserror(const char *mes)
+{
+	perror(mes);
+}
+
+
 /*****************************************************************************************
  *
  * Prompt.
@@ -75,7 +97,7 @@ static int initPrompt()
 	/* Get current directory. */
 	if(getcwd(crntDir,CRNT_DIR_BUF_SIZE)==NULL)
 	{
-		printf("Failed getcwd\n");
+		printf("shell : Failed getcwd\n");
 		return -1;
 	}
 
@@ -85,6 +107,112 @@ static int initPrompt()
 }
 
 
+/*****************************************************************************************
+ *
+ * Job control.
+ *
+ ******************************************************************************************/
+
+
+/*
+ * PUBLIC
+ * ֥ꥹȡ
+ */
+static pid_t volatile job[CHILD_MAX];
+static int jobCnt;
+
+
+/*
+ * return : 0 or max=1
+ */
+static int chkJobNum()
+{
+	if(jobCnt>0)
+	{
+		while(getpgid(job[jobCnt-1])==-1)
+			job[--jobCnt]=0;
+
+		if(jobCnt==CHILD_MAX)return 1;
+	}
+	return 0;
+}
+
+
+/*
+ * ֤Ͽ
+ */
+static void addJob(pid_t pid)
+{
+	job[jobCnt++]=pid;
+}
+
+
+/*
+ * ֤κ
+ */
+static void delJob()
+{
+	int i;
+
+
+	for(i=0;i<jobCnt;++i)
+		if(getpgid(job[i])==-1)
+		{
+			memcpy((void*)&job[i],(void*)&job[i+1],(jobCnt-i-1)*sizeof(pid_t));
+			job[--jobCnt]=0;
+			--i;
+		}
+}
+
+
+/*
+ * fg
+ *  '%n(nϿ)' Τб
+ */
+static void fg(char **arg)
+{
+	int num;
+	pid_t grp;
+
+
+	/* γǧ */
+	if(arg[0]==NULL)
+	{
+		if((num=jobCnt-1)<0)return errstr("No job!");
+		while((grp=getpgid(job[num]))<0)
+		{
+			job[num]=0;
+			if(--num<0)return errstr("No job!");
+		}
+		jobCnt=num+1;
+	}
+	else if(*arg[0]=='%')
+	{
+		if(((num=atoi(&arg[0][1]))==0)||(num>CHILD_MAX))
+			return errstr("Argument is wrong!");
+		num=jobCnt-num;
+		if((job[num]==0)||((grp=getpgid(job[num]))==-1))
+		{
+			memcpy((void*)&job[num],(void*)&job[num+1],(jobCnt-num-1)*sizeof(pid_t));
+			job[--jobCnt]=0;
+			return errstr("Argument is wrong!");
+		}
+	}
+	else return errstr("Argument is wrong!");
+
+	/* ե饦ɥ롼פѹ */
+	if(tcsetpgrp(0,grp)==-1)
+		return syserror("Failed tcsetpgrp()");
+	kill(-grp,SIGCONT);					/* ե饦ɥ롼פSIGCONT롣 */
+
+	/* ȡ */
+	waitpid(job[num],NULL,0);
+
+	if((grp=getpgrp())!=tcgetpgrp(0))
+		tcsetpgrp(0,grp);			/* üե饦ɥ롼פ᤹ */
+}
+
+
 /*****************************************************************************************
  *
  * Execute file.
@@ -92,13 +220,6 @@ static int initPrompt()
  ******************************************************************************************/
 
 
-/*
- * PUBLIC
- * ֥롼ץꥹȡ
- */
-static pid_t volatile job[CHILD_MAX];
-
-
 /*
  * PUBLIC
  * Execute file.
@@ -107,20 +228,14 @@ static pid_t volatile job[CHILD_MAX];
 static void execFile(char **arg)
 {
 	int pid;
-	int num;
 	int backgrnd=0;		/* Хå饦ɼ¹ԥե饰 */
 	int state=0;
 	int i;
 
 
-	/* ¹Իҥץå */
-	num=0;
-	while(job[num]!=0)
-		if(++num==CHILD_MAX)
-		{
-			printf("Number of child processes is max!");
-			return;
-		}
+	/* ֿå */
+	if(chkJobNum()==1)
+		return errstr("shell : Number of child processes is max!");
 
 	/* '&'õ */
 	for(i=1;arg[i]!=NULL;++i)
@@ -131,6 +246,7 @@ static void execFile(char **arg)
 			break;
 		}
 
+	/* ¹ԡ */
 	pid=fork();
     if(pid==0)
 	{
@@ -144,7 +260,7 @@ static void execFile(char **arg)
 		return;
 	}
 
-	job[num]=pid;
+	addJob(pid);
 	if(backgrnd==0)
 	{
 		pid_t grp;
@@ -158,11 +274,8 @@ static void execFile(char **arg)
 		if(WIFEXITED(state))
 		{
 			if(WEXITSTATUS(state)==1)
-				printf("%s don't execute!\n",arg[0]);
-
-			/* ֥ơ֥뤫 */
-			memcpy((void*)&job[num],(void*)&job[num+1],CHILD_MAX-num-1);
-			job[CHILD_MAX-1]=0;
+				printf("shell : %s execute error!\n",arg[0]);
+			delJob();
 		}
 	}
 }
@@ -180,21 +293,6 @@ typedef struct{
 }COMMAND;
 
 
-/*
- * PRIVATE
- * 顼ɽ
- */
-static void errstr(const char *mes)
-{
-	printf("%s\n",mes);
-}
-
-static void syserror(const char *mes)
-{
-	perror(mes);
-}
-
-
 /*
  * PRIVATE
  * ʸκǸ夬'/'ʤ'\0'񤭤롣
@@ -230,54 +328,7 @@ static void cd(char **arg)
 /* reboot */
 static void _reboot(char **arg)
 {
-	if(reboot(REBOOT_REBOOT)==-1)printf("Failed halt\n");
-}
-
-/*
- * fg
- *  '%n(nϿ)' Τб
- */
-static void fg(char **arg)
-{
-	uint num;
-	pid_t grp;
-
-
-	/* γǧ */
-	if(arg[0]==NULL)num=0;
-	else if(*arg[0]=='%')
-	{
-		if(((num=atoi(&arg[0][1]))==0)||(num>CHILD_MAX))
-			return errstr("Argument is wrong!");
-		num-=1;
-	}
-	else return errstr("Argument is wrong!");
-
-	/* ե饦ɥ롼פѹ */
-	for(;;)
-	{
-		if(job[num]==0)
-			return errstr("Argument is wrong!");
-		else
-		{
-			if((grp=getpgid(job[num]))<0)
-				memcpy((void*)&job[num],(void*)&job[num+1],CHILD_MAX-num-1);
-			else
-			{
-				if(tcsetpgrp(0,grp)==-1)
-					return syserror("Failed tcsetpgrp()");
-				kill(-grp,SIGCONT);					/* ե饦ɥ롼פSIGCONT롣 */
-				
-				break;
-			}
-		}
-	}
-
-	/* ȡ */
-	waitpid(job[num],NULL,0);
-
-	if((grp=getpgrp())!=tcgetpgrp(0))
-		tcsetpgrp(0,grp);			/* üե饦ɥ롼פ᤹ */
+	if(reboot(REBOOT_REBOOT)==-1)printf("shell : Failed halt\n");
 }
 
 /* halt */
@@ -357,7 +408,7 @@ static void execCommand(char *arg[])
 		else high=middle-1;
 	}
 	if((low>=CMD_NUM)||(strcmp(arg[0],command[low].name)!=0))
-		printf("Command ""%s"" not found\n",arg[0]);
+		printf("shell : Command ""%s"" not found\n",arg[0]);
 	else command[low].cmd(&arg[1]);
 }
 
@@ -435,7 +486,7 @@ int main()
 	/* ץץȤɽ */
 	if(initPrompt()!=0)
 	{
-		printf("Fialed init prompt!\n");
+		printf("shell : Fialed init prompt!\n");
 		return 0;
 	}
 
@@ -444,9 +495,9 @@ int main()
 		printf("%s>",crntDir);
 
 		/* üϼա */
-		if((size=read(STDIN_FILENO,buf,ARG_SIZE-1))!=0)
+		if((size=read(STDIN_FILENO,buf,ARG_SIZE-1))>1)
 		{
-			buf[size]='\0';
+			buf[size-1]='\0';
 
 			/* ȡʬ䡣 */
 			parseCommandLine(buf,arg);
@@ -455,6 +506,11 @@ int main()
 			if(strchr(arg[0],'/')!=NULL)execFile(arg);
 			else execCommand(arg);
 		}
+		else if(size<0)
+		{
+			syserror("shell : failed read standard in");
+			break;
+		}
 	}
 
 	return 0;
