@@ -39,6 +39,8 @@ typedef struct WAIT_INTR{
 }WAIT_INTR;
 
 
+extern void waitProc(PROC_LINK*);
+extern void wakeProc(PROC_LINK*);
 extern void waitSemaphore(WAIT_QUEUE*);
 extern void wakeSemaphore(WAIT_QUEUE*);
 extern int wait_intr(WAIT_INTR*,uint,int);
